# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from typing import Any
from typing import Callable
from typing import Dict
from typing import Optional

from ansible.errors import AnsibleError
from ansible.module_utils.common.text.converters import to_bytes

from ansible_collections.amazon.aws.plugins.plugin_utils.ssm.common import CommandResult
from ansible_collections.amazon.aws.plugins.plugin_utils.ssm.common import ssm_retry


class FileTransferManager:
    """Manages file transfers using S3 as an intermediary."""

    def __init__(
        self,
        bucket_name: str,
        instance_id: str,
        s3_client,
        reconnection_retries: int,
        verbosity_display: Callable[[int, str], None],
        close: Callable[[], None],
        exec_command: Callable[[str, Optional[bool], Optional[bool]], CommandResult],
    ):
        """
        Initializes the FileTransferManager with a given connection.

        :param connection: The connection object used for S3 and remote execution.
        """
        self.bucket_name = bucket_name
        self.instance_id = instance_id
        self.s3_client = s3_client
        self.reconnection_retries = reconnection_retries
        self.verbosity_display = verbosity_display
        self.close = close
        self.exec_command = exec_command

    @ssm_retry
    def _file_transport_command(
        self,
        in_path: str,
        out_path: str,
        ssm_action: str,
        command: str,
        put_args: Dict[str, Any],
        s3_path: str,
    ) -> CommandResult:
        """
        Transfers files to or from a remote host using S3.

        :param in_path: The local path of the file to be transferred.
        :param out_path: The remote path where the file should be placed.
        :param ssm_action: The action to perform, either 'get' (download) or 'put' (upload).
        :param command: A shell command string
        :param put_args: Additional arguments for S3 upload.
        :param s3_path: The S3 path where the file is stored.
        :return: A CommandResult dictionary containing execution results.
        """
        try:
            if ssm_action == "get":
                return self._handle_get(in_path, out_path, command, s3_path)
            return self._handle_put(in_path, out_path, command, s3_path, put_args)
        finally:
            self.s3_client.delete_object(Bucket=self.bucket_name, Key=s3_path)

    def _handle_get(self, in_path: str, out_path: str, command: str, s3_path: str) -> CommandResult:
        """
        Handles downloading a file from the remote host via S3.

        :param in_path: The local destination path for the file.
        :param out_path: The remote source path of the file.
        :param commands: The transport commands to execute.
        :param s3_path: The S3 path where the file is stored.
        :return: A CommandResult dictionary containing execution results.
        """

        result = self._exec_transport_commands(in_path, out_path, command)
        with open(to_bytes(out_path, errors="surrogate_or_strict"), "wb") as data:
            self.s3_client.download_fileobj(self.bucket_name, s3_path, data)

        return result

    def _handle_put(
        self, in_path: str, out_path: str, command: str, s3_path: str, put_args: Dict[str, Any]
    ) -> CommandResult:
        """
        Handles uploading a file to the remote host via S3.

        :param in_path: The local source path of the file.
        :param out_path: The remote destination path for the file.
        :param commands: The transport commands to execute.
        :param s3_path: The S3 path where the file will be stored.
        :param put_args: Additional arguments for S3 upload.
        :return: A CommandResult dictionary containing execution results.
        """

        with open(to_bytes(in_path, errors="surrogate_or_strict"), "rb") as data:
            self.s3_client.upload_fileobj(data, self.bucket_name, s3_path, ExtraArgs=put_args)

        result = self._exec_transport_commands(in_path, out_path, command)

        return result

    def _exec_transport_commands(self, in_path: str, out_path: str, command: str) -> CommandResult:
        """
        Executes the provided transport commands.

        :param in_path: The input path.
        :param out_path: The output path.
        :param commands: A list of command dictionaries containing the command string and metadata.
        :return: A CommandResult dictionary containing execution results.
        """
        returncode, stdout, stderr = self.exec_command(command, in_data=None, sudoable=False)

        # Check the return code
        if returncode != 0:
            raise AnsibleError(f"failed to transfer file to {in_path} {out_path}:\n{stdout}\n{stderr}")

        return {"returncode": returncode, "stdout": stdout, "stderr": stderr}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
from ansible.plugins.action import ActionBase
try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator, )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.meraki.plugins.plugin_utils.meraki import (
    MERAKI,
    meraki_argument_spec,
    meraki_compare_equality2,
    get_dict_result,
)
from ansible_collections.cisco.meraki.plugins.plugin_utils.exceptions import (
    InconsistentParameters,
)

# Get common arguments specification
argument_spec = meraki_argument_spec()
# Add arguments specific for this module
argument_spec.update(dict(
    state=dict(type="str", default="present", choices=["present"]),
    enforcePasswordExpiration=dict(type="bool"),
    passwordExpirationDays=dict(type="int"),
    enforceDifferentPasswords=dict(type="bool"),
    numDifferentPasswords=dict(type="int"),
    enforceStrongPasswords=dict(type="bool"),
    minimumPasswordLength=dict(type="int"),
    enforceAccountLockout=dict(type="bool"),
    accountLockoutAttempts=dict(type="int"),
    enforceIdleTimeout=dict(type="bool"),
    idleTimeoutMinutes=dict(type="int"),
    enforceTwoFactorAuth=dict(type="bool"),
    enforceLoginIpRanges=dict(type="bool"),
    loginIpRanges=dict(type="list"),
    apiAuthentication=dict(type="dict"),
    organizationId=dict(type="str"),
))

required_if = [
    ("state", "present", ["organizationId"], True),
]
required_one_of = []
mutually_exclusive = []
required_together = []


class OrganizationsLoginSecurity(object):
    def __init__(self, params, meraki):
        self.meraki = meraki
        self.new_object = dict(
            enforcePasswordExpiration=params.get("enforcePasswordExpiration"),
            passwordExpirationDays=params.get("passwordExpirationDays"),
            enforceDifferentPasswords=params.get("enforceDifferentPasswords"),
            numDifferentPasswords=params.get("numDifferentPasswords"),
            enforceStrongPasswords=params.get("enforceStrongPasswords"),
            minimumPasswordLength=params.get("minimumPasswordLength"),
            enforceAccountLockout=params.get("enforceAccountLockout"),
            accountLockoutAttempts=params.get("accountLockoutAttempts"),
            enforceIdleTimeout=params.get("enforceIdleTimeout"),
            idleTimeoutMinutes=params.get("idleTimeoutMinutes"),
            enforceTwoFactorAuth=params.get("enforceTwoFactorAuth"),
            enforceLoginIpRanges=params.get("enforceLoginIpRanges"),
            loginIpRanges=params.get("loginIpRanges"),
            apiAuthentication=params.get("apiAuthentication"),
            organizationId=params.get("organizationId"),
        )

    def get_all_params(self, name=None, id=None):
        new_object_params = {}
        if self.new_object.get('organizationId') is not None or self.new_object.get(
                'organization_id') is not None:
            new_object_params['organizationId'] = self.new_object.get(
                'organizationId') or self.new_object.get('organization_id')
        return new_object_params

    def update_all_params(self):
        new_object_params = {}
        if self.new_object.get('enforcePasswordExpiration') is not None or self.new_object.get(
                'enforce_password_expiration') is not None:
            new_object_params['enforcePasswordExpiration'] = self.new_object.get(
                'enforcePasswordExpiration')
        if self.new_object.get('passwordExpirationDays') is not None or self.new_object.get(
                'password_expiration_days') is not None:
            new_object_params['passwordExpirationDays'] = self.new_object.get(
                'passwordExpirationDays') or self.new_object.get('password_expiration_days')
        if self.new_object.get('enforceDifferentPasswords') is not None or self.new_object.get(
                'enforce_different_passwords') is not None:
            new_object_params['enforceDifferentPasswords'] = self.new_object.get(
                'enforceDifferentPasswords')
        if self.new_object.get('numDifferentPasswords') is not None or self.new_object.get(
                'num_different_passwords') is not None:
            new_object_params['numDifferentPasswords'] = self.new_object.get(
                'numDifferentPasswords') or self.new_object.get('num_different_passwords')
        if self.new_object.get('enforceStrongPasswords') is not None or self.new_object.get(
                'enforce_strong_passwords') is not None:
            new_object_params['enforceStrongPasswords'] = self.new_object.get(
                'enforceStrongPasswords')
        if self.new_object.get('minimumPasswordLength') is not None or self.new_object.get(
                'minimum_password_length') is not None:
            new_object_params['minimumPasswordLength'] = self.new_object.get(
                'minimumPasswordLength') or self.new_object.get('minimum_password_length')
        if self.new_object.get('enforceAccountLockout') is not None or self.new_object.get(
                'enforce_account_lockout') is not None:
            new_object_params['enforceAccountLockout'] = self.new_object.get(
                'enforceAccountLockout')
        if self.new_object.get('accountLockoutAttempts') is not None or self.new_object.get(
                'account_lockout_attempts') is not None:
            new_object_params['accountLockoutAttempts'] = self.new_object.get(
                'accountLockoutAttempts') or self.new_object.get('account_lockout_attempts')
        if self.new_object.get('enforceIdleTimeout') is not None or self.new_object.get(
                'enforce_idle_timeout') is not None:
            new_object_params['enforceIdleTimeout'] = self.new_object.get(
                'enforceIdleTimeout')
        if self.new_object.get('idleTimeoutMinutes') is not None or self.new_object.get(
                'idle_timeout_minutes') is not None:
            new_object_params['idleTimeoutMinutes'] = self.new_object.get(
                'idleTimeoutMinutes') or self.new_object.get('idle_timeout_minutes')
        if self.new_object.get('enforceTwoFactorAuth') is not None or self.new_object.get(
                'enforce_two_factor_auth') is not None:
            new_object_params['enforceTwoFactorAuth'] = self.new_object.get(
                'enforceTwoFactorAuth')
        if self.new_object.get('enforceLoginIpRanges') is not None or self.new_object.get(
                'enforce_login_ip_ranges') is not None:
            new_object_params['enforceLoginIpRanges'] = self.new_object.get(
                'enforceLoginIpRanges')
        if self.new_object.get('loginIpRanges') is not None or self.new_object.get(
                'login_ip_ranges') is not None:
            new_object_params['loginIpRanges'] = self.new_object.get(
                'loginIpRanges') or self.new_object.get('login_ip_ranges')
        if self.new_object.get('apiAuthentication') is not None or self.new_object.get(
                'api_authentication') is not None:
            new_object_params['apiAuthentication'] = self.new_object.get(
                'apiAuthentication') or self.new_object.get('api_authentication')
        if self.new_object.get('organizationId') is not None or self.new_object.get(
                'organization_id') is not None:
            new_object_params['organizationId'] = self.new_object.get(
                'organizationId') or self.new_object.get('organization_id')
        return new_object_params

    def get_object_by_name(self, name):
        result = None
        # NOTE: Does not have a get by name method, using get all
        try:
            items = self.meraki.exec_meraki(
                family="organizations",
                function="getOrganizationLoginSecurity",
                params=self.get_all_params(name=name),
            )
            if isinstance(items, dict):
                if 'response' in items:
                    items = items.get('response')
            result = get_dict_result(items, 'name', name)
            if result is None:
                result = items
        except Exception as e:
            print("Error: ", e)
            result = None
        return result

    def get_object_by_id(self, id):
        result = None
        # NOTE: Does not have a get by id method or it is in another action
        return result

    def exists(self):
        prev_obj = None
        id_exists = False
        name_exists = False
        o_id = self.new_object.get("id")
        name = self.new_object.get("name")
        if o_id:
            prev_obj = self.get_object_by_name(o_id)
            id_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if not id_exists and name:
            prev_obj = self.get_object_by_name(name)
            name_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if name_exists:
            _id = prev_obj.get("id")
            if id_exists and name_exists and o_id != _id:
                raise InconsistentParameters(
                    "The 'id' and 'name' params don't refer to the same object")
            if _id:
                self.new_object.update(dict(id=_id))
        it_exists = prev_obj is not None and isinstance(prev_obj, dict)
        return (it_exists, prev_obj)

    def requires_update(self, current_obj):
        requested_obj = self.new_object

        obj_params = [
            ("enforcePasswordExpiration", "enforcePasswordExpiration"),
            ("enforceDifferentPasswords", "enforceDifferentPasswords"),
            ("numDifferentPasswords", "numDifferentPasswords"),
            ("enforceStrongPasswords", "enforceStrongPasswords"),
            ("minimumPasswordLength", "minimumPasswordLength"),
            ("enforceAccountLockout", "enforceAccountLockout"),
            ("accountLockoutAttempts", "accountLockoutAttempts"),
            ("enforceIdleTimeout", "enforceIdleTimeout"),
            ("idleTimeoutMinutes", "idleTimeoutMinutes"),
            ("enforceTwoFactorAuth", "enforceTwoFactorAuth"),
            ("enforceLoginIpRanges", "enforceLoginIpRanges"),
            ("loginIpRanges", "loginIpRanges"),
            ("apiAuthentication", "apiAuthentication"),]
        # Method 1. Params present in request (Ansible) obj are the same as the current (ISE) params
        # If any does not have eq params, it requires update
        return any(
            not meraki_compare_equality2(
                current_obj.get(meraki_param),
                requested_obj.get(ansible_param)) for (
                meraki_param,
                ansible_param) in obj_params)

    def update(self):
        id = self.new_object.get("id")
        name = self.new_object.get("name")
        result = None
        result = self.meraki.exec_meraki(
            family="organizations",
            function="updateOrganizationLoginSecurity",
            params=self.update_all_params(),
            op_modifies=True,
        )
        return result


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail(
                "ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'")
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        meraki = MERAKI(self._task.args)
        obj = OrganizationsLoginSecurity(self._task.args, meraki)

        state = self._task.args.get("state")

        response = None
        if state == "present":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                if obj.requires_update(prev_obj):
                    response = obj.update()
                    meraki.object_updated()
                else:
                    response = prev_obj
                    meraki.object_already_present()
            else:
                meraki.fail_json(
                    "Object does not exists, plugin only has update")

        self._result.update(dict(meraki_response=response))
        self._result.update(meraki.exit_json())
        return self._result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: cloudformation_info
version_added: 1.0.0
short_description: Obtain information about an AWS CloudFormation stack
description:
  - Gets information about an AWS CloudFormation stack.
author:
    - Justin Menga (@jmenga)
    - Kevin Coming (@waffie1)
options:
    stack_name:
        description:
          - The name or id of the CloudFormation stack. Gathers information on all stacks by default.
        type: str
    all_facts:
        description:
            - Get all stack information for the stack.
        type: bool
        default: false
    stack_events:
        description:
            - Get stack events for the stack.
        type: bool
        default: false
    stack_template:
        description:
            - Get stack template body for the stack.
        type: bool
        default: false
    stack_resources:
        description:
            - Get stack resources for the stack.
        type: bool
        default: false
    stack_policy:
        description:
            - Get stack policy for the stack.
        type: bool
        default: false
    stack_change_sets:
        description:
            - Get stack change sets for the stack
        type: bool
        default: false
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Get information on all stacks
  amazon.aws.cloudformation_info:
  register: all_stacks_output

- name: Get summary information about a stack
  amazon.aws.cloudformation_info:
    stack_name: my-cloudformation-stack
  register: output

- debug:
    msg: "{{ output['cloudformation']['my-cloudformation-stack'] }}"

# Get stack outputs, when you have the stack name available as a fact
- set_fact:
    stack_name: my-awesome-stack

- amazon.aws.cloudformation_info:
    stack_name: "{{ stack_name }}"
  register: my_stack

- debug:
    msg: "{{ my_stack.cloudformation[stack_name].stack_outputs }}"

# Get all stack information about a stack
- amazon.aws.cloudformation_info:
    stack_name: my-cloudformation-stack
    all_facts: true

# Get stack resource and stack policy information about a stack
- amazon.aws.cloudformation_info:
    stack_name: my-cloudformation-stack
    stack_resources: true
    stack_policy: true

# Fail if the stack doesn't exist
- name: try to get info about a stack but fail if it doesn't exist
  amazon.aws.cloudformation_info:
    stack_name: nonexistent-stack
    all_facts: true
  failed_when: cloudformation['nonexistent-stack'] is undefined
"""

RETURN = r"""
cloudformation:
    description:
        - Dictionary of dictionaries containing info of stack(s).
        - Keys are stack_name(s).
    returned: always
    type: dict
    contains:
        stack_description:
            description: Summary facts about the stack.
            returned: if the stack exists
            type: dict
            contains:
                capabilities:
                    description: The capabilities allowed in the stack.
                    returned: always
                    type: list
                    elements: str
                creation_time:
                    description: The time at which the stack was created.
                    returned: if stack exists
                    type: str
                deletion_time:
                    description: The time at which the stack was deleted.
                    returned: if stack was deleted
                    type: str
                description:
                    description: The user-defined description associated with the stack.
                    returned: always
                    type: str
                disable_rollback:
                    description: Whether or not rollback on stack creation failures is enabled.
                    returned: always
                    type: bool
                drift_information:
                    description: Information about whether a stack's actual configuration differs, or has drifted, from it's expected configuration,
                        as defined in the stack template and any values specified as template parameters.
                    returned: always
                    type: dict
                    contains:
                        stack_drift_status:
                            description: Status of the stack's actual configuration compared to its expected template configuration.
                            returned: always
                            type: str
                        last_check_timestamp:
                            description: Most recent time when a drift detection operation was initiated on the stack,
                                or any of its individual resources that support drift detection.
                            returned: if a drift was detected
                            type: str
                enable_termination_protection:
                    description: Whether termination protection is enabled for the stack.
                    returned: always
                    type: bool
                notification_arns:
                    description: Amazon SNS topic ARNs to which stack related events are published.
                    returned: always
                    type: list
                    elements: str
                outputs:
                    description: A list of output dicts.
                    returned: always
                    type: list
                    elements: dict
                    contains:
                        output_key:
                            description: The key associated with the output.
                            returned: always
                            type: str
                        output_value:
                            description: The value associated with the output.
                            returned: always
                            type: str
                parameters:
                    description: A list of parameter dicts.
                    returned: always
                    type: list
                    elements: dict
                    contains:
                        parameter_key:
                            description: The key associated with the parameter.
                            returned: always
                            type: str
                        parameter_value:
                            description: The value associated with the parameter.
                            returned: always
                            type: str
                rollback_configuration:
                    description: The rollback triggers for CloudFormation to monitor during stack creation and updating operations.
                    returned: always
                    type: dict
                    contains:
                        rollback_triggers:
                            description: The triggers to monitor during stack creation or update actions.
                            returned: when rollback triggers exist
                            type: list
                            elements: dict
                            contains:
                                arn:
                                    description: The ARN of the rollback trigger.
                                    returned: always
                                    type: str
                                type:
                                    description: The resource type of the rollback trigger.
                                    returned: always
                                    type: str
                stack_id:
                    description: The unique ID of the stack.
                    returned: always
                    type: str
                stack_name:
                    description: The name of the stack.
                    returned: always
                    type: str
                stack_status:
                    description: The status of the stack.
                    returned: always
                    type: str
                tags:
                    description: A list of tags associated with the stack.
                    returned: always
                    type: list
                    elements: dict
                    contains:
                        key:
                            description: Key of tag.
                            returned: always
                            type: str
                        value:
                            description: Value of tag.
                            returned: always
                            type: str
        stack_outputs:
            description: Dictionary of stack outputs keyed by the value of each output 'OutputKey' parameter and corresponding value of each
                        output 'OutputValue' parameter.
            returned: if the stack exists
            type: dict
            sample: { ApplicationDatabaseName: dazvlpr01xj55a.ap-southeast-2.rds.amazonaws.com }
        stack_parameters:
            description: Dictionary of stack parameters keyed by the value of each parameter 'ParameterKey' parameter and corresponding value of
                        each parameter 'ParameterValue' parameter.
            returned: if the stack exists
            type: dict
            sample:
                {
                    DatabaseEngine: mysql,
                    DatabasePassword: "***"
                }
        stack_events:
            description: All stack events for the stack.
            returned: only if O(all_facts) or O(stack_events) is V(true) and the stack exists
            type: list
        stack_policy:
            description: Describes the stack policy for the stack.
            returned: only if O(all_facts) or O(stack_policy) is V(true) and the stack exists
            type: dict
        stack_template:
            description: Describes the stack template for the stack.
            returned: only if O(all_facts) or O(stack_policy) is V(true) and the stack exists
            type: dict
        stack_resource_list:
            description: Describes stack resources for the stack.
            returned: only if O(all_facts) or O(stack_policy) is V(true) and the stack exists
            type: list
        stack_resources:
            description: Dictionary of stack resources keyed by the value of each resource 'LogicalResourceId' parameter and corresponding value of each
                        resource 'PhysicalResourceId' parameter.
            returned: only if O(all_facts) or O(stack_policy) is V(true) and the stack exists
            type: dict
            sample: {
                "AutoScalingGroup": "dev-someapp-AutoscalingGroup-1SKEXXBCAN0S7",
                "AutoScalingSecurityGroup": "sg-abcd1234",
                "ApplicationDatabase": "dazvlpr01xj55a"
            }
        stack_change_sets:
            description: A list of stack change sets.  Each item in the list represents the details of a specific changeset.
            returned: only if O(all_facts) or O(stack_policy) is V(true) and the stack exists
            type: list
        stack_tags:
            description: Dictionary of key value pairs of tags.
            returned: only if O(all_facts) or O(stack_policy) is V(true) and the stack exists
            type: dict
            sample: {
                'TagOne': 'ValueOne',
                'TagTwo': 'ValueTwo'
            }
"""

import json

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_message
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict


class CloudFormationServiceManager:
    """Handles CloudFormation Services"""

    def __init__(self, module):
        self.module = module
        self.client = module.client("cloudformation")

    @AWSRetry.exponential_backoff(retries=5, delay=5)
    def describe_stacks_with_backoff(self, **kwargs):
        paginator = self.client.get_paginator("describe_stacks")
        return paginator.paginate(**kwargs).build_full_result()["Stacks"]

    def describe_stacks(self, stack_name=None):
        try:
            kwargs = {"StackName": stack_name} if stack_name else {}
            response = self.describe_stacks_with_backoff(**kwargs)
            if response is not None:
                return response
            self.module.fail_json(msg="Error describing stack(s) - an empty response was returned")
        except is_boto3_error_message("does not exist"):
            return {}
        except (
            botocore.exceptions.BotoCoreError,
            botocore.exceptions.ClientError,
        ) as e:  # pylint: disable=duplicate-except
            self.module.fail_json_aws(e, msg="Error describing stack " + stack_name)

    @AWSRetry.exponential_backoff(retries=5, delay=5)
    def list_stack_resources_with_backoff(self, stack_name):
        paginator = self.client.get_paginator("list_stack_resources")
        return paginator.paginate(StackName=stack_name).build_full_result()["StackResourceSummaries"]

    def list_stack_resources(self, stack_name):
        try:
            return self.list_stack_resources_with_backoff(stack_name)
        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
            self.module.fail_json_aws(e, msg="Error listing stack resources for stack " + stack_name)

    @AWSRetry.exponential_backoff(retries=5, delay=5)
    def describe_stack_events_with_backoff(self, stack_name):
        paginator = self.client.get_paginator("describe_stack_events")
        return paginator.paginate(StackName=stack_name).build_full_result()["StackEvents"]

    def describe_stack_events(self, stack_name):
        try:
            return self.describe_stack_events_with_backoff(stack_name)
        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
            self.module.fail_json_aws(e, msg="Error listing stack events for stack " + stack_name)

    @AWSRetry.exponential_backoff(retries=5, delay=5)
    def list_stack_change_sets_with_backoff(self, stack_name):
        paginator = self.client.get_paginator("list_change_sets")
        return paginator.paginate(StackName=stack_name).build_full_result()["Summaries"]

    @AWSRetry.exponential_backoff(retries=5, delay=5)
    def describe_stack_change_set_with_backoff(self, **kwargs):
        paginator = self.client.get_paginator("describe_change_set")
        return paginator.paginate(**kwargs).build_full_result()

    def describe_stack_change_sets(self, stack_name):
        changes = []
        try:
            change_sets = self.list_stack_change_sets_with_backoff(stack_name)
            for item in change_sets:
                changes.append(
                    self.describe_stack_change_set_with_backoff(
                        StackName=stack_name, ChangeSetName=item["ChangeSetName"]
                    )
                )
            return changes
        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
            self.module.fail_json_aws(e, msg="Error describing stack change sets for stack " + stack_name)

    @AWSRetry.exponential_backoff(retries=5, delay=5)
    def get_stack_policy_with_backoff(self, stack_name):
        return self.client.get_stack_policy(StackName=stack_name)

    def get_stack_policy(self, stack_name):
        try:
            response = self.get_stack_policy_with_backoff(stack_name)
            stack_policy = response.get("StackPolicyBody")
            if stack_policy:
                return json.loads(stack_policy)
            return dict()
        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
            self.module.fail_json_aws(e, msg="Error getting stack policy for stack " + stack_name)

    @AWSRetry.exponential_backoff(retries=5, delay=5)
    def get_template_with_backoff(self, stack_name):
        return self.client.get_template(StackName=stack_name)

    def get_template(self, stack_name):
        try:
            response = self.get_template_with_backoff(stack_name)
            return response.get("TemplateBody")
        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
            self.module.fail_json_aws(e, msg="Error getting stack template for stack " + stack_name)


def to_dict(items, key, value):
    """Transforms a list of items to a Key/Value dictionary"""
    if items:
        return dict(zip([i.get(key) for i in items], [i.get(value) for i in items]))
    else:
        return dict()


def main():
    argument_spec = dict(
        stack_name=dict(),
        all_facts=dict(required=False, default=False, type="bool"),
        stack_policy=dict(required=False, default=False, type="bool"),
        stack_events=dict(required=False, default=False, type="bool"),
        stack_resources=dict(required=False, default=False, type="bool"),
        stack_template=dict(required=False, default=False, type="bool"),
        stack_change_sets=dict(required=False, default=False, type="bool"),
    )
    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    service_mgr = CloudFormationServiceManager(module)

    result = {"cloudformation": {}}

    for stack_description in service_mgr.describe_stacks(module.params.get("stack_name")):
        facts = {"stack_description": stack_description}
        stack_name = stack_description.get("StackName")

        # Create stack output and stack parameter dictionaries
        if facts["stack_description"]:
            facts["stack_outputs"] = to_dict(facts["stack_description"].get("Outputs"), "OutputKey", "OutputValue")
            facts["stack_parameters"] = to_dict(
                facts["stack_description"].get("Parameters"), "ParameterKey", "ParameterValue"
            )
            facts["stack_tags"] = boto3_tag_list_to_ansible_dict(facts["stack_description"].get("Tags"))

        # Create optional stack outputs
        all_facts = module.params.get("all_facts")
        if all_facts or module.params.get("stack_resources"):
            facts["stack_resource_list"] = service_mgr.list_stack_resources(stack_name)
            facts["stack_resources"] = to_dict(
                facts.get("stack_resource_list"), "LogicalResourceId", "PhysicalResourceId"
            )
        if all_facts or module.params.get("stack_template"):
            facts["stack_template"] = service_mgr.get_template(stack_name)
        if all_facts or module.params.get("stack_policy"):
            facts["stack_policy"] = service_mgr.get_stack_policy(stack_name)
        if all_facts or module.params.get("stack_events"):
            facts["stack_events"] = service_mgr.describe_stack_events(stack_name)
        if all_facts or module.params.get("stack_change_sets"):
            facts["stack_change_sets"] = service_mgr.describe_stack_change_sets(stack_name)

        result["cloudformation"][stack_name] = camel_dict_to_snake_dict(
            facts,
            ignore_list=(
                "stack_outputs",
                "stack_parameters",
                "stack_policy",
                "stack_resources",
                "stack_tags",
                "stack_template",
            ),
        )
    module.exit_json(changed=False, **result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_adaptive_policy_acls
short_description: Resource module for organizations _adaptive _policy _acls
description:
  - Manage operations create, update and delete of the resource organizations _adaptive
    _policy _acls.
  - Creates new adaptive policy ACL.
  - Deletes the specified adaptive policy ACL. Note this adaptive policy ACL will
    also be removed from policies using it.
  - Updates an adaptive policy ACL.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  aclId:
    description: AclId path parameter. Acl ID.
    type: str
  description:
    description: Description of the adaptive policy ACL.
    type: str
  ipVersion:
    description: IP version of adpative policy ACL. One of 'any', 'ipv4' or 'ipv6'.
    type: str
  name:
    description: Name of the adaptive policy ACL.
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  rules:
    description: An ordered array of the adaptive policy ACL rules.
    elements: dict
    suboptions:
      dstPort:
        description: Destination port. Must be in the format of single port '1', port
          list '1,2' or port range '1-10', and in the range of 1-65535, or 'any'.
          Default is 'any'.
        type: str
      log:
        description: If enabled, when this rule is hit an entry will be logged to
          the event log.
        type: bool
      policy:
        description: '''allow'' or ''deny'' traffic specified by this rule.'
        type: str
      protocol:
        description: The type of protocol (must be 'tcp', 'udp', 'icmp' or 'any').
        type: str
      srcPort:
        description: Source port. Must be in the format of single port '1', port list
          '1,2' or port range '1-10', and in the range of 1-65535, or 'any'. Default
          is 'any'.
        type: str
      tcpEstablished:
        description: If enabled, means TCP connection with this node must be established.
          Can only be enabled when policy protocol is 'allow tcp'.
        type: bool
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations createOrganizationAdaptivePolicyAcl
    description: Complete reference of the createOrganizationAdaptivePolicyAcl API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-organization-adaptive-policy-acl
  - name: Cisco Meraki documentation for organizations deleteOrganizationAdaptivePolicyAcl
    description: Complete reference of the deleteOrganizationAdaptivePolicyAcl API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-adaptive-policy-acl
  - name: Cisco Meraki documentation for organizations updateOrganizationAdaptivePolicyAcl
    description: Complete reference of the updateOrganizationAdaptivePolicyAcl API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-adaptive-policy-acl
notes:
  - SDK Method used are
    organizations.Organizations.create_organization_adaptive_policy_acl,
    organizations.Organizations.delete_organization_adaptive_policy_acl,
    organizations.Organizations.update_organization_adaptive_policy_acl,
  - Paths used are
    post /organizations/{organizationId}/adaptivePolicy/acls,
    delete
    /organizations/{organizationId}/adaptivePolicy/acls/{aclId},
    put /organizations/{organizationId}/adaptivePolicy/acls/{aclId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_adaptive_policy_acls:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    description: Blocks sensitive web traffic
    ipVersion: ipv6
    name: Block sensitive web traffic
    organizationId: string
    rules:
      - dstPort: 22-30
        log: true
        policy: deny
        protocol: tcp
        srcPort: 1,33
        tcpEstablished: true
- name: Update by id
  cisco.meraki.organizations_adaptive_policy_acls:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    aclId: string
    description: Blocks sensitive web traffic
    ipVersion: ipv6
    name: Block sensitive web traffic
    organizationId: string
    rules:
      - dstPort: 22-30
        log: true
        policy: deny
        protocol: tcp
        srcPort: 1,33
        tcpEstablished: true
- name: Delete by id
  cisco.meraki.organizations_adaptive_policy_acls:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    aclId: string
    organizationId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "aclId": "string",
      "name": "string",
      "description": "string",
      "ipVersion": "string",
      "rules": [
        {
          "policy": "string",
          "protocol": "string",
          "srcPort": "string",
          "dstPort": "string",
          "log": true,
          "tcpEstablished": true
        }
      ],
      "createdAt": "string",
      "updatedAt": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2022 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_aksagentpool
version_added: '1.14.0'
short_description: Manage node pools in Kubernetes kubernetes cluster
description:
    - Create, update or delete node pools in kubernetes cluster.

options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: True
    cluster_name:
        description:
            - The name of the kubernetes cluster.
        type: str
        required: True
    name:
        description:
            - The name of the node agent pool.
        type: str
        required: True
    count:
        description:
            - Number of agents (VMs) to host docker containers.
        type: int
    vm_size:
        description:
            - Size of agent VMs
        type: str
    os_disk_size_gb:
        description:
            - OS Disk Size in GB to be used to specify the disk size for every machine in this master/agent pool.
        type: int
    vnet_subnet_id:
        description:
            - VNet SubnetID specifies the VNet's subnet identifier.
        type: str
    availability_zones:
        description:
            - Availability zones for nodes. Must use VirtualMachineScaleSets AgentPoolType.
        type: list
        elements: int
        choices:
            - 1
            - 2
            - 3
    os_type:
        description:
            - OsType to be used to specify os type.
        type: str
        choices:
            - Linux
            - Windows
            - linux
            - windows
    orchestrator_version:
        description:
            - Version of orchestrator specified when creating the managed cluster.
        type: str
    type_properties_type:
        description:
            - AgentPoolType represents types of an agent pool.
        type: str
        choices:
            - VirtualMachineScaleSets
            - AvailabilitySet
    mode:
        description:
            - AgentPoolMode represents mode of an agent pool.
        type: str
        choices:
            - System
            - User
    enable_auto_scaling:
        description:
            - Whether to enable auto-scaler.
        type: bool
    max_count:
        description:
            - Maximum number of nodes for auto-scaling.
        type: int
    node_labels:
        description:
            -  Agent pool node labels to be persisted across all nodes in agent pool.
        type: dict
    node_taints:
        description:
            - The taints added to new nodes during node pool create and scale.
            - For example as example, value:NoSchedule'.
        type: list
        elements: str
    min_count:
        description:
            - Minimum number of nodes for auto-scaling.
        type: int
    max_pods:
        description:
            - Maximum number of pods that can run on a node.
        type: int
    kubelet_disk_type:
        description:
            - Determines the placement of emptyDir volumes, container runtime data root, and Kubelet ephemeral storage.
        type: str
        choices:
            - OS
            - Temporary
    workload_runtime:
        description:
            - Determines the type of workload a node can run.
        type: str
        choices:
            - OCIContainer
            - WasmWasi
    os_sku:
        description:
            - Specifies an OS SKU.
            - This value must not be specified if OSType is Windows.
            - The I(os_sku=CBLMariner) deprecated. Microsoft recommends that new deployments choose 'AzureLinux' instead.
        type: str
        choices:
            - Ubuntu
            - AzureLinux
            - Windows2019
            - Windows2022
    scale_down_mode:
        description:
            - This also effects the cluster autoscaler behavior.
            - If not specified, it defaults to C(Delete).
        type: str
        default: Delete
        choices:
            - Delete
            - Deallocate
    upgrade_settings:
        description:
            - Settings for upgrading the agentpool.
        type: dict
        suboptions:
            max_surge:
                description:
                    - This can either be set to an integer, sucha as C(5) or percentage C(50%).
                    - If a percentage is specified, it is the percentage of the total agent pool size at the time of the upgrade.
                    - For percentages, fractional nodes are rounded up.
                    - If not specified, the default is C(1).
                type: str
    power_state:
        description:
            - When an Agent Pool is first created it is initially C(Running).
            - The Agent Pool can be stopped by setting this field to C(Stopped).
            -  A stopped Agent Pool stops all of its VMs and does not accrue billing charges.
            - An Agent Pool can only be stopped if it is Running and provisioning state is Succeeded.
        type: dict
        suboptions:
            code:
                description:
                    - Tells whether the cluster is C(Running) or C(Stopped).
                type: str
                choices:
                    - Running
                    - Stopped
    enable_node_public_ip:
        description:
            - Some scenarios may require nodes in a node pool to receive theirown dedicated public IP addresses.
            - A common scenario is for gaming workloads, where a console needs to make a direct connection to a cloud virtual machine to minimize hops.
        type: bool
    scale_set_priority:
        description:
            - The Virtual Machine Scale Set priority.
            - If not specified, the default is C(Regular).
        type: str
        choices:
            - Spot
            - Regular
    node_public_ip_prefix_id:
        description:
            - The Azure Public IP prefix's ID.
        type: str
    scale_set_eviction_policy:
        description:
            - This cannot be specified unless the I(scale_set_priority=Spot).
            - If not specified, the default is C(Delete).
        type: str
        choices:
            - Delete
            - Deallocate
    spot_max_price:
        description:
            - Possible values are any decimal value greater than zero or -1.
            - Indicates the willingness to pay any on-demand price.
        type: float
    proximity_placement_group_id:
        description:
            - The ID for Proximity Placement Group.
        type: str
    kubelet_config:
        description:
            - The Kubelet configuration on the agent pool nodes.
        type: dict
        suboptions:
            cpu_manager_policy:
                description:
                    - Kubernetes CPU management policies.
                    - The default is C(none).
                type: str
                default: none
                choices:
                    - none
                    - static
            cpu_cfs_quota:
                description:
                    - The default is C(true).
                type: bool
                default: true
            cpu_cfs_quota_period:
                description:
                    - The default is C(100ms).
                    - Valid values are a sequence of decimal numbers with an optional fraction and a unit suffix.
                type: str
                default: 100ms
            image_gc_high_threshold:
                description:
                    - To disable image garbage collection, set to C(100).
                    - The default is C(85)
                type: int
                default: 85
            image_gc_low_threshold:
                description:
                    - This cannot be set higher than imageGcHighThreshold.
                    - The default is C(80).
                type: int
                default: 80
            topology_manager_policy:
                description:
                    - Kubernetes Topology Manager policies.
                    - The default is C(none).
                type: str
                default: none
                choices:
                    - none
                    - best-effort
                    - restricted
                    - single-numa-node
            allowed_unsafe_sysctls:
                description:
                    - Allowed list of unsafe sysctls or unsafe sysctl patterns.
                type: list
                elements: str
            fail_swap_on:
                description:
                    - If set to true it will make the Kubelet fail to start if swap is enabled on the node.
                type: bool
            container_log_max_size_mb:
                description:
                    - The maximum size of container log file before it is rotated.
                type: int
            container_log_max_files:
                description:
                    - The maximum number of container log files that can be present for a container. The number must be ≥ 2.
                type: int
            pod_max_pids:
                description:
                    - The maximum number of processes per pod.
                type: int
    linux_os_config:
        description:
            - The OS configuration of Linux agent nodes.
        type: dict
        suboptions:
            sysctls:
                description:
                    - Sysctl settings for Linux agent nodes.
                type: dict
                suboptions:
                    net_core_somaxconn:
                        description:
                            - Sysctl setting net.core.somaxconn.
                        type: int
                    net_core_netdev_max_backlog:
                        description:
                            - Sysctl setting net.core.netdev_max_backlog.
                        type: int
                    net_core_rmem_default:
                        description:
                            - Sysctl setting net.core.rmem_default.
                        type: int
                    net_core_rmem_max:
                        description:
                            - Sysctl setting net.core.rmem_max.
                        type: int
                    net_core_wmem_default:
                        description:
                            - Sysctl setting net.core.wmem_default.
                        type: int
                    net_core_wmem_max:
                        description:
                            - Sysctl setting net.core.wmem_max.
                        type: int
                    net_core_optmem_max:
                        description:
                            - Sysctl setting net.core.optmem_max.
                        type: int
                    net_ipv4_tcp_max_syn_backlog:
                        description:
                            - Sysctl setting net.ipv4.tcp_max_syn_backlog.
                        type: int
                    net_ipv4_tcp_max_tw_buckets:
                        description:
                            - Sysctl setting net.ipv4.tcp_max_tw_buckets.
                        type: int
                    net_ipv4_tcp_fin_timeout:
                        description:
                            - Sysctl setting net.ipv4.tcp_fin_timeout.
                        type: int
                    net_ipv4_tcp_keepalive_time:
                        description:
                            - Sysctl setting net.ipv4.tcp_keepalive_time.
                        type: int
                    net_ipv4_tcp_keepalive_probes:
                        description:
                            - Sysctl setting net.ipv4.tcp_keepalive_probes.
                        type: int
                    net_ipv4_tcpkeepalive_intvl:
                        description:
                            - Sysctl setting net.ipv4.tcp_keepalive_intvl.
                        type: int
                    net_ipv4_tcp_tw_reuse:
                        description:
                            - Sysctl setting net.ipv4.tcp_tw_reuse.
                        type: bool
                    net_ipv4_ip_local_port_range:
                        description:
                            - Sysctl setting net.ipv4.ip_local_port_range.
                        type: str
                    net_ipv4_neigh_default_gc_thresh1:
                        description:
                            - Sysctl setting net.ipv4.neigh.default.gc_thresh1.
                        type: int
                    net_ipv4_neigh_default_gc_thresh2:
                        description:
                            - Sysctl setting net.ipv4.neigh.default.gc_thresh2.
                        type: int
                    net_ipv4_neigh_default_gc_thresh3:
                        description:
                            - Sysctl setting net.ipv4.neigh.default.gc_thresh3.
                        type: int
                    fs_inotify_max_user_watches:
                        description:
                            - Sysctl setting fs.inotify.max_user_watches.
                        type: int
                    fs_file_max:
                        description:
                            - Sysctl setting fs.file-max.
                        type: int
                    fs_aio_max_nr:
                        description:
                            - Sysctl setting fs.aio-max-nr.
                        type: int
                    fs_nr_open:
                        description:
                            - Sysctl setting fs.nr_open.
                        type: int
                    kernel_threads_max:
                        description:
                            - Sysctl setting kernel.threads-max.
                        type: int
                    vm_max_map_count:
                        description:
                            - Sysctl setting vm.max_map_count.
                        type: int
                    vm_swappiness:
                        description:
                            - Sysctl setting vm.swappiness.
                        type: int
                    vm_vfs_cache_pressure:
                        description:
                            - Sysctl setting vm.vfs_cache_pressure.
                        type: int
                    net_netfilter_nf_conntrack_max:
                        description:
                            - sysctl setting net.netfilter.nf_conntrack_max.
                        type: int
                    net_netfilter_nf_conntrack_buckets:
                        description:
                            - Sysctl setting net.netfilter.nf_conntrack_buckets.
                        type: int
            transparent_huge_page_enabled:
                description:
                    - The node agent pool transparent hugepage.
                    - The default is C(always).
                type: str
                default: always
                choices:
                    - always
                    - madvise
                    - never
            transparent_huge_page_defrag:
                description:
                    - The node agent pool transparent huge page deferag.
                    - The default is C(madvise).
                type: str
                default: madvise
                choices:
                    - always
                    - defer
                    - defer+madvise
                    - madvise
                    - never
            swap_file_size_mb:
                description:
                    - The size in MB of a swap file that will be created on each node.
                type: int
    enable_encryption_at_host:
        description:
            - This is only supported on certain VM sizes and in certain Azure regions.
        type: bool
    enable_ultra_ssd:
        description:
            - Whether to enable UltraSSD.
        type: bool
    enable_fips:
        description:
            - Whether enable FIPS node pool.
        type: bool
    gpu_instance_profile:
        description:
            - GPUInstanceProfile to be used to specify GPU MIG instance profile for supported GPU VM SKU.
        type: str
        choices:
            - MIG1g
            - MIG2g
            - MIG3g
            - MIG4g
            - MIG7g
    security_profile:
        description:
            - The security settings of an agent pool.
        type: dict
        suboptions:
            enable_vtpm:
                description:
                    - Whether to disable or enabled the vTPM.
                type: bool
                default: false
            enable_secure_boot:
                description:
                    - Whether to disable or enabled the secure boot.
                default: false
                type: bool
    os_disk_type:
        description:
            - The default is C(Ephemeral) if the VM supports it and has a cache disk larger than the requested OSDiskSizeGB.
            - Otherwise, defaults to C(Managed).
            - May not be changed after creation.
        type: str
        choices:
            - Ephemeral
            - Managed
    capacity_reservation_group_id:
        description:
            - The Capacity Reservation Group ID.
        type: str
    host_group_id:
        description:
            - The Host Group ID.
            - Sample as C(/subscriptions/{subscriptionId}/resourceGroups/{RG_Name}/providers/Microsoft.Compute/hostGroups/{hostGroupName)
        type: str
    pod_subnet_id:
        description:
            - If omitted, pod IPs are statically assigned on the node subnet.
        type: str
    windows_profile:
        description:
            - The Windows agent pool's specific profile.
        type: dict
        suboptions:
            disable_outbound_nat:
                description:
                    - The default value is C(false).
                    - Outbound NAT can only be disabled if the cluster outboundType is NAT Gateway.
                    - The Windows agent pool does not have node public IP enabled.
                type: bool
    network_profile:
        description:
            - Network-related settings of an agent pool.
        type: dict
        suboptions:
            node_public_ip_tags:
                description:
                    - IPTags of instance-level public IPs.
                type: list
                elements: dict
                suboptions:
                    ip_tag_type:
                        description:
                            - The IP tag type. Example as C(RoutingPreference).
                        type: str
                    tag:
                        description:
                            - The value of the IP tag associated with the public IP. Sample as C(Internet).
                        type: str
            allowed_host_ports:
                description:
                    - The port ranges that are allowed to access.
                    - The specified ranges are allowed to overlap.
                type: list
                elements: dict
                suboptions:
                    port_start:
                        description:
                            - The minimum port that is included in the range.
                            - It should be ranged from C(1) to C(65535), and be less than or equal to I(port_end).
                        type: int
                    port_end:
                        description:
                            - The maximum port that is included in the range.
                            - It should be ranged from C(1) to C(65535), and be greater than or equal to I(port_start).
                        type: int
                    protocol:
                        description:
                            - The network protocol of the port.
                        type: str
                        choices:
                            - UDP
                            - TCP
            application_security_groups:
                description:
                    - The IDs of the application security groups which agent pool will associate when created.
                type: list
                elements: str
    state:
        description:
            - State of the automation runbook. Use C(present) to create or update a automation runbook and use C(absent) to delete.
        type: str
        default: present
        choices:
            - present
            - absent

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Add new node agent pool
  azure_rm_aksagentpool:
    resource_group: "{{ resource_group }}"
    cluster_name: aksfred01
    name: default-new
    count: 2
    vm_size: Standard_B2s
    type_properties_type: VirtualMachineScaleSets
    mode: System
    node_labels: {"release":"stable"}
    max_pods: 42
    orchestrator_version: 1.23.5
    availability_zones:
      - 1
      - 2
- name: Create a node agent pool with custom config
  azure_rm_aksagentpool:
    resource_group: "{{ resource_group }}"
    cluster_name: "min{{ rpfx }}"
    name: default-new2
    count: 1
    vm_size: Standard_B2s
    type_properties_type: VirtualMachineScaleSets
    mode: System
    node_labels: {"release":"stable"}
    max_pods: 42
    enable_auto_scaling: true
    min_count: 1
    max_count: 10
    orchestrator_version: 1.23.5
    availability_zones:
      - 1
    kubelet_config:
      cpu_manager_policy: static
      cpu_cfs_quota: true
      fail_swap_on: false
    linux_os_config:
      transparent_huge_page_enabled: madvise
      swap_file_size_mb: 1500
      transparent_huge_page_defrag: defer+madvise
- name: Delete node agent pool
  azure_rm_aksagentpool:
    resource_group: "{{ resource_group }}"
    cluster_name: aksfred01
    name: default-new
'''

RETURN = '''
aks_agent_pools:
    description:
        - Details for a node pool in the managed Kubernetes cluster.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            type: str
            returned: always
            sample: "/subscriptions/xxx-xxxf/resourcegroups/myRG/providers/Microsoft.ContainerService/managedClusters/cluster/agentPools/default"
        resource_group:
            description:
                - Resource group name.
            type: str
            returned: always
            sample: myRG
        name:
            description:
                - Resource name.
            type: str
            returned: always
            sample: default
        cluster_name:
            description:
                - The cluster name.
            type: str
            returned: always
            sample: testcluster
        availability_zones:
            description:
                - Availability zones for nodes. Must use VirtualMachineScaleSets AgentPoolType.
            type: list
            returned: always
            sample: [1, 2]
        count:
            description:
                - Number of agents (VMs) to host docker containers.
            type: int
            returned: always
            sample: 2
        enable_auto_scaling:
            description:
                - Whether to enable auto-scaler.
            type: str
            returned: always
            sample: null
        enable_node_public_ip:
            description:
                -  Enable public IP for nodes.
            type: str
            returned: always
            sample: bool
        max_count:
            description:
                - Maximum number of nodes for auto-scaling.
            type: int
            returned: always
            sample: 10
        min_count:
            description:
                - Minimum number of nodes for auto-scaling.
            type: int
            returned: always
            sample: 1
        max_pods:
            description:
                - Maximum number of pods that can run on a node.
            type: int
            returned: always
            sample: 42
        mode:
            description:
                - AgentPoolMode represents mode of an agent pool.
            type: str
            returned: always
            sample: System
        node_image_version:
            description:
                - Version of node image.
            type: str
            returned: always
            sample: AKSUbuntu-1804gen2containerd-2022.08.23
        node_labels:
            description:
                - Agent pool node labels to be persisted across all nodes in agent pool.
            type: list
            returned: always
            sample: ["release": "stable"]
        node_taints:
            description:
                - Taints added to new nodes during node pool create and scale.
            type: list
            returned: always
            sample: ["CriticalAddonsOnly=false:NoSchedule"]
        orchestrator_version:
            description:
                - Version of orchestrator specified when creating the managed cluster.
            type: str
            returned: always
            sample: 1.22.11
        os_disk_size_gb:
            description:
                - OS Disk Size in GB to be used to specify the disk size for every machine in this master/agent pool.
            type: int
            returned: always
            sample: 128
        os_type:
            description:
                - OsType to be used to specify os type.
            type: str
            returned: always
            sample: Linux
        provisioning_state:
            description:
                - The current deployment or provisioning state, which only appears in the response.
            type: str
            returned: always
            sample: Succeeded
        scale_set_eviction_policy:
            description:
                - ScaleSetEvictionPolicy to be used to specify eviction policy for Spot virtual machine scale set.
            type: str
            returned: always
            sample: null
        scale_set_priority:
            description:
                - caleSetPriority to be used to specify virtual machine scale set priority.
            type: str
            returned: always
            sample: null
        spot_max_price:
            description:
                - SpotMaxPrice to be used to specify the maximum price you are willing to pay in US Dollars.
            type: float
            returned: always
            sample: null
        security_profile:
            description:
                - The security settings of an agent pool.
            type: complex
            contains:
                enable_secure_boot:
                    description:
                        - The secure boot is disabled or enabled.
                    type: bool
                    returned: always
                    sample: true
                enable_vtpm:
                    description:
                        - The vTPM is enabled or disabled.
                    type: bool
                    returned: always
                    sample: true
        type:
            description:
                - Resource Type.
            type: str
            returned: always
            sample: Microsoft.ContainerService/managedClusters/agentPools
        type_properties_type:
            description:
                - AgentPoolType represents types of an agent pool.
            type: str
            returned: always
            sample: VirtualMachineScaleSets
        upgrade_settings:
            description:
                - Settings for upgrading the agentpool.
            type: str
            returned: always
            sample: null
        vm_size:
            description:
                - Size of agent VMs.
            type: str
            returned: always
            sample: Standard_B2s
        vnet_subnet_id:
            description:
                - VNet SubnetID specifies the VNet's subnet identifier.
            type: str
            returned: always
            sample: null
        kubelet_config:
            description:
                - The Kubelet configuration on the agent pool nodes.
            type: dict
            returned: always
            sample: {
                    cpu_cfs_quota: true,
                    cpu_cfs_quota_period: 100ms,
                    cpu_manager_policy: static,
                    fail_swap_on: false,
                    image_gc_high_threshold: 85,
                    image_gc_low_threshold: 80,
                    topology_manager_policy: none
                }
        linux_os_config:
            description:
                - The OS configuration of Linux agent nodes.
            type: dict
            returned: always
            sample: {
                    swap_file_size_mb: 1500,
                    sysctls: {},
                    transparent_huge_page_defrag: defer+madvise,
                    transparent_huge_page_enabled: madvise
                }
        power_state:
            description:
                - The agent pool's power state.
            type: dict
            returned: always
            sample: {code: Running}
        os_sku:
            description:
                - The node agent pool's SKU.
            type: str
            returned: always
            sample: Ubuntu
        tags:
            description:
                - The tags of the node agent pool.
            type: dict
            returned: always
            sample: {key1: value1, key2: value2}
        kubelet_disk_type:
            description:
                - Determines the placement of emptyDir volumes, container runtime data root, and Kubelet ephemeral storage.
            type: str
            returned: always
            sample: OS
        workload_runtime:
            description:
                - Determines the type of workload a node can run.
            type: str
            returned: always
            sample: OCIContainer
        scale_down_mode:
            description:
                - This also effects the cluster autoscaler behavior.
            type: str
            returned: always
            sample: Delete
        node_public_ip_prefix_id:
            description:
                - The Azure Public IP prefix's ID.
            type: str
            returned: always
            sample: null
        proximity_placement_group_id:
            description:
                - The ID for Proximity Placement Group.
            type: str
            returned: always
            sample: /subscriptions/xxx-xxx/resourceGroups/myRG/providers/Microsoft.Compute/proximityPlacementGroups/proxi01
        enable_encryption_at_host:
            description:
                - This is only supported on certain VM sizes and in certain Azure regions.
            type: bool
            returned: always
            sample: false
        enable_ultra_ssd:
            description:
                - Whether enable FIPS node pool.
            type: bool
            returned: always
            sample: false
        gpu_instance_profile:
            description:
                - GPUInstanceProfile to be used to specify GPU MIG instance profile for supported GPU VM SKU.
            type: str
            returned: always
            sample: MIG1g
        windows_profile:
            description:
                - The Windows agent pool's specific profile.
            type: dict
            returned: when-used
            sample: {"disable_outbound_nat": false}
        network_profile:
            description:
                - Network-related settings of an agent pool.
            type: complex
            returned: when-used
            contains:
                allowed_host_ports:
                    description:
                        - The port ranges that are allowed to access.
                    type: list
                    returned: always
                    sample: [{"port_end": 200, "port_start": 10, "protocol": "TCP"}]
                application_security_groups:
                    description:
                        - The IDs of the application security groups which agent pool will associate when created.
                    type: list
                    returned: always
                    sample: ["/subscriptions/xxxxxx/resourceGroups/testRG/providers/Microsoft.Network/applicationSecurityGroups/appnsg"]
        pod_subnet_id:
            description:
                - The subnet ID.
            type: str
            returned: always
            sample: "/subscriptions/xxxx/resourceGroups/testRG/providers/Microsoft.Network/virtualNetworks/My_Virtual_Network/subnets/foobar"
        host_group_id:
            description:
                - The host group ID.
            type: str
            returned: always
            sample: "/subscriptions/xxxxx/resourceGroups/testRG/providers/Microsoft.Compute/hostGroups/hostgroup"
        capacity_reservation_group_id:
            description:
                - The ID of Capacity Reservation Group.
            returned: always
            type: str
            sample: "/subscriptions/xxxxx/resourceGroups/testRG/providers/Microsoft.Compute/capacityReservationGroups/crgid"
        os_disk_type:
            description:
                - The type of the OS disk.
            returned: always
            sample: Managed
            type: str
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    pass


class AzureRMAksAgentPool(AzureRMModuleBaseExt):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            cluster_name=dict(
                type='str',
                required=True
            ),
            count=dict(
                type='int',
            ),
            vm_size=dict(
                type='str',
            ),
            os_disk_size_gb=dict(
                type='int'
            ),
            vnet_subnet_id=dict(
                type='str'
            ),
            availability_zones=dict(
                type='list',
                elements='int',
                choices=[1, 2, 3]
            ),
            os_type=dict(
                type='str',
                choices=['Linux', 'Windows', 'linux', 'windows']
            ),
            orchestrator_version=dict(
                type='str',
            ),
            type_properties_type=dict(
                type='str',
                choices=['VirtualMachineScaleSets', 'AvailabilitySet']
            ),
            mode=dict(
                type='str',
                choices=['System', 'User'],
            ),
            enable_auto_scaling=dict(
                type='bool'
            ),
            max_count=dict(
                type='int'
            ),
            node_labels=dict(
                type='dict'
            ),
            node_taints=dict(
                type='list',
                elements='str'
            ),
            min_count=dict(
                type='int'
            ),
            max_pods=dict(
                type='int'
            ),
            kubelet_disk_type=dict(
                type='str', choices=['OS', 'Temporary']
            ),
            workload_runtime=dict(
                type='str', choices=['OCIContainer', 'WasmWasi']
            ),
            os_sku=dict(
                type='str', choices=["Ubuntu", "AzureLinux", "Windows2022", "Windows2019"]
            ),
            scale_down_mode=dict(
                type='str',
                choices=['Delete', 'Deallocate'],
                default='Delete'
            ),
            upgrade_settings=dict(
                type='dict',
                options=dict(
                    max_surge=dict(
                        type='str'
                    )
                )
            ),
            power_state=dict(
                type='dict',
                options=dict(
                    code=dict(
                        type='str',
                        choices=['Running', 'Stopped']
                    )
                )
            ),
            enable_node_public_ip=dict(
                type='bool'
            ),
            scale_set_priority=dict(
                type='str',
                choices=["Spot", "Regular"],
            ),
            node_public_ip_prefix_id=dict(
                type='str'
            ),
            scale_set_eviction_policy=dict(
                type='str',
                choices=['Delete', 'Deallocate'],
            ),
            spot_max_price=dict(
                type='float'
            ),
            proximity_placement_group_id=dict(
                type='str'
            ),
            kubelet_config=dict(
                type='dict',
                options=dict(
                    cpu_manager_policy=dict(type='str', choices=['none', 'static'], default='none'),
                    cpu_cfs_quota=dict(type='bool', default='true'),
                    cpu_cfs_quota_period=dict(type='str', default='100ms'),
                    image_gc_high_threshold=dict(type='int', default=85),
                    image_gc_low_threshold=dict(type='int', default=80),
                    topology_manager_policy=dict(
                        type='str',
                        default='none',
                        choices=['none', 'best-effort', 'restricted', 'single-numa-node']
                    ),
                    allowed_unsafe_sysctls=dict(
                        type='list',
                        elements='str'
                    ),
                    fail_swap_on=dict(type='bool'),
                    container_log_max_size_mb=dict(type='int'),
                    container_log_max_files=dict(type='int'),
                    pod_max_pids=dict(type='int')
                )
            ),
            linux_os_config=dict(
                type='dict',
                options=dict(
                    sysctls=dict(
                        type='dict',
                        options=dict(
                            net_core_somaxconn=dict(type='int'),
                            net_core_netdev_max_backlog=dict(type='int'),
                            net_core_rmem_default=dict(type='int'),
                            net_core_rmem_max=dict(type='int'),
                            net_core_wmem_default=dict(type='int'),
                            net_core_wmem_max=dict(type='int'),
                            net_core_optmem_max=dict(type='int'),
                            net_ipv4_tcp_max_syn_backlog=dict(type='int'),
                            net_ipv4_tcp_max_tw_buckets=dict(type='int'),
                            net_ipv4_tcp_fin_timeout=dict(type='int'),
                            net_ipv4_tcp_keepalive_time=dict(type='int'),
                            net_ipv4_tcp_keepalive_probes=dict(type='int'),
                            net_ipv4_tcpkeepalive_intvl=dict(type='int'),
                            net_ipv4_tcp_tw_reuse=dict(type='bool'),
                            net_ipv4_ip_local_port_range=dict(type='str'),
                            net_ipv4_neigh_default_gc_thresh1=dict(type='int'),
                            net_ipv4_neigh_default_gc_thresh2=dict(type='int'),
                            net_ipv4_neigh_default_gc_thresh3=dict(type='int'),
                            net_netfilter_nf_conntrack_max=dict(type='int'),
                            net_netfilter_nf_conntrack_buckets=dict(type='int'),
                            fs_inotify_max_user_watches=dict(type='int'),
                            fs_file_max=dict(type='int'),
                            fs_aio_max_nr=dict(type='int'),
                            fs_nr_open=dict(type='int'),
                            kernel_threads_max=dict(type='int'),
                            vm_max_map_count=dict(type='int'),
                            vm_swappiness=dict(type='int'),
                            vm_vfs_cache_pressure=dict(type='int')
                        )
                    ),
                    transparent_huge_page_enabled=dict(
                        type='str',
                        choices=['always', 'madvise', 'never'],
                        default='always'
                    ),
                    swap_file_size_mb=dict(
                        type='int'
                    ),
                    transparent_huge_page_defrag=dict(
                        type='str',
                        default='madvise',
                        choices=['always', 'defer', 'defer+madvise', 'madvise', 'never']
                    )
                )
            ),
            enable_encryption_at_host=dict(
                type='bool'
            ),
            enable_ultra_ssd=dict(
                type='bool'
            ),
            enable_fips=dict(
                type='bool'
            ),
            gpu_instance_profile=dict(
                type='str',
                choices=["MIG1g", "MIG2g", "MIG3g", "MIG4g", "MIG7g"]
            ),
            security_profile=dict(
                type='dict',
                options=dict(
                    enable_secure_boot=dict(type='bool', default=False),
                    enable_vtpm=dict(type='bool', default=False)
                )
            ),
            os_disk_type=dict(
                type='str',
                choices=['Managed', 'Ephemeral']
            ),
            capacity_reservation_group_id=dict(
                type='str',
            ),
            host_group_id=dict(
                type='str',
            ),
            pod_subnet_id=dict(
                type='str',
            ),
            network_profile=dict(
                type='dict',
                options=dict(
                    node_public_ip_tags=dict(
                        type='list',
                        elements='dict',
                        options=dict(
                            ip_tag_type=dict(type='str'),
                            tag=dict(type='str')
                        )
                    ),
                    allowed_host_ports=dict(
                        type='list',
                        elements='dict',
                        options=dict(
                            port_start=dict(type='int'),
                            port_end=dict(type='int'),
                            protocol=dict(
                                type='str',
                                choices=['UDP', 'TCP']
                            )
                        )
                    ),
                    application_security_groups=dict(
                        type='list',
                        elements='str'
                    )
                )
            ),
            windows_profile=dict(
                type='dict',
                options=dict(
                    disable_outbound_nat=dict(type='bool')
                )
            ),
            state=dict(
                type='str',
                choices=['present', 'absent'],
                default='present'
            )
        )
        # store the results of the module operation
        self.results = dict()
        self.resource_group = None
        self.name = None
        self.cluster_name = None
        self.count = None
        self.vm_size = None
        self.mode = None
        self.os_disk_size_gb = None
        self.storage_profiles = None
        self.vnet_subnet_id = None
        self.availability_zones = None
        self.os_type = None
        self.orchestrator_version = None
        self.type_properties_type = None
        self.enable_auto_scaling = None
        self.max_count = None
        self.node_labels = None
        self.node_taints = None
        self.min_count = None
        self.max_pods = None
        self.tags = None
        self.kubelet_disk_type = None
        self.workload_runtime = None
        self.os_sku = None
        self.scale_down_mode = None
        self.upgrade_settings = None
        self.power_state = None
        self.enable_node_public_ip = None
        self.scale_set_priority = None
        self.node_public_ip_prefix_id = None
        self.scale_set_eviction_policy = None
        self.spot_max_price = None
        self.proximity_placement_group_id = None
        self.kubelet_config = None
        self.linux_os_config = None
        self.enable_encryption_at_host = None
        self.enable_ultra_ssd = None
        self.enable_fips = None
        self.gpu_instance_profile = None
        self.security_profile = None
        self.os_disk_type = None
        self.capacity_reservation_group_id = None
        self.host_group_id = None
        self.pod_subnet_id = None
        self.network_profile = None
        self.windows_profile = None
        self.body = dict()

        super(AzureRMAksAgentPool, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec) + ['tags']:
            setattr(self, key, kwargs[key])
            if key not in ['resource_group', 'cluster_name', 'name', 'state']:
                self.body[key] = kwargs[key]

        agent_pool = self.get()
        changed = False
        response = None

        if self.state == 'present':
            if agent_pool:
                update_tags, self.body['tags'] = self.update_tags(agent_pool.get('tags'))
                for key in self.body.keys():
                    if key == 'tags':
                        if update_tags:
                            changed = True
                    elif self.body[key] is not None and isinstance(self.body[key], dict):
                        for item in self.body[key].keys():
                            if key == 'security_profile':
                                if bool(self.body[key][item]) != bool(agent_pool[key].get(item)):
                                    changed = True
                            elif key == 'windows_profile':
                                if agent_pool.get(key) is None or bool(self.body[key][item]) != bool(agent_pool[key].get(item)):
                                    changed = True
                            elif not self.default_compare({}, self.body[key], agent_pool[key], '', dict(compare=[])):
                                changed = True
                    elif key == 'node_public_ip_prefix_id':
                        pass
                    elif key == 'node_taints' and self.body[key] is not None:
                        if len(self.body[key]) != len(agent_pool[key]) or (not all(item in agent_pool[key] for item in self.body[key])):
                            changed = True
                    elif self.body[key] is not None and self.body[key] != agent_pool[key] and key not in ['scale_set_priority', 'spot_max_price']:
                        changed = True
                    else:
                        self.body[key] = agent_pool[key]
            else:
                changed = True

            if changed:
                if not self.check_mode:
                    response = self.create_or_update(self.body)

        else:
            if not self.check_mode:
                if agent_pool:
                    response = self.delete_agentpool()
                    changed = True
                else:
                    changed = False
            else:
                changed = True

        self.results['changed'] = changed
        self.results['aks_agent_pools'] = response
        return self.results

    def get(self):
        try:
            response = self.managedcluster_client.agent_pools.get(self.resource_group, self.cluster_name, self.name)
            return self.to_dict(response)
        except ResourceNotFoundError:
            pass

    def create_or_update(self, parameters):
        try:
            response = self.managedcluster_client.agent_pools.begin_create_or_update(self.resource_group, self.cluster_name, self.name, parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
            return self.to_dict(response)
        except Exception as exc:
            self.fail('Error when creating cluster node agent pool {0}: {1}'.format(self.name, exc))

    def delete_agentpool(self):
        try:
            self.managedcluster_client.agent_pools.begin_delete(self.resource_group, self.cluster_name, self.name)
        except Exception as exc:
            self.fail('Error when deleting cluster agent pool {0}: {1}'.format(self.name, exc))

    def to_dict(self, agent_pool):
        if not agent_pool:
            return None
        agent_pool_dict = dict(
            resource_group=self.resource_group,
            cluster_name=self.cluster_name,
            id=agent_pool.id,
            type=agent_pool.type,
            name=agent_pool.name,
            count=agent_pool.count,
            vm_size=agent_pool.vm_size,
            os_disk_size_gb=agent_pool.os_disk_size_gb,
            vnet_subnet_id=agent_pool.vnet_subnet_id,
            max_pods=agent_pool.max_pods,
            os_type=agent_pool.os_type,
            max_count=agent_pool.max_count,
            min_count=agent_pool.min_count,
            enable_auto_scaling=agent_pool.enable_auto_scaling,
            type_properties_type=agent_pool.type_properties_type,
            mode=agent_pool.mode,
            orchestrator_version=agent_pool.orchestrator_version,
            node_image_version=agent_pool.node_image_version,
            upgrade_settings=dict(),
            provisioning_state=agent_pool.provisioning_state,
            availability_zones=[],
            enable_node_public_ip=agent_pool.enable_node_public_ip,
            scale_set_priority=agent_pool.scale_set_priority,
            scale_set_eviction_policy=agent_pool.scale_set_eviction_policy,
            spot_max_price=agent_pool.spot_max_price,
            node_labels=agent_pool.node_labels,
            node_taints=agent_pool.node_taints if agent_pool.node_taints else [],
            tags=agent_pool.tags,
            kubelet_disk_type=agent_pool.kubelet_disk_type,
            workload_runtime=agent_pool.workload_runtime,
            os_sku=agent_pool.os_sku,
            scale_down_mode=agent_pool.scale_down_mode,
            power_state=dict(),
            node_public_ip_prefix_id=agent_pool.node_public_ip_prefix_id,
            proximity_placement_group_id=agent_pool.proximity_placement_group_id,
            kubelet_config=dict(),
            linux_os_config=dict(),
            security_profile=dict(),
            enable_encryption_at_host=agent_pool.enable_encryption_at_host,
            enable_ultra_ssd=agent_pool.enable_ultra_ssd,
            enable_fips=agent_pool.enable_fips,
            gpu_instance_profile=agent_pool.gpu_instance_profile,
            os_disk_type=agent_pool.os_disk_type,
            capacity_reservation_group_id=agent_pool.capacity_reservation_group_id,
            host_group_id=agent_pool.host_group_id,
            pod_subnet_id=agent_pool.pod_subnet_id,
            network_profile=dict(),
            windows_profile=dict(),
        )

        if agent_pool.upgrade_settings is not None:
            agent_pool_dict['upgrade_settings']['max_surge'] = agent_pool.upgrade_settings.max_surge
        else:
            agent_pool_dict['upgrade_settings'] = None

        if agent_pool.availability_zones is not None:
            for key in agent_pool.availability_zones:
                agent_pool_dict['availability_zones'].append(int(key))
        else:
            agent_pool_dict['availability_zones'] = None

        if agent_pool.kubelet_config is not None:
            agent_pool_dict['kubelet_config'] = agent_pool.kubelet_config.as_dict()
        else:
            agent_pool_dict['kubelet_config'] = None

        if agent_pool.power_state is not None:
            agent_pool_dict['power_state']['code'] = agent_pool.power_state.code
        else:
            agent_pool_dict['power_state'] = None

        if agent_pool.linux_os_config is not None:
            agent_pool_dict['linux_os_config']['transparent_huge_page_enabled'] = agent_pool.linux_os_config.transparent_huge_page_enabled
            agent_pool_dict['linux_os_config']['transparent_huge_page_defrag'] = agent_pool.linux_os_config.transparent_huge_page_defrag
            agent_pool_dict['linux_os_config']['swap_file_size_mb'] = agent_pool.linux_os_config.swap_file_size_mb
            agent_pool_dict['linux_os_config']['sysctls'] = dict()
            if agent_pool.linux_os_config.sysctls is not None:
                agent_pool_dict['linux_os_config']['sysctls'] = agent_pool.linux_os_config.sysctls.as_dict()
            else:
                agent_pool_dict['linux_os_config']['sysctls'] = None
        else:
            agent_pool_dict['linux_os_config'] = None

        if agent_pool.security_profile is not None:
            agent_pool_dict['security_profile']['enable_vtpm'] = agent_pool.security_profile.enable_vtpm
            agent_pool_dict['security_profile']['enable_secure_boot'] = agent_pool.security_profile.enable_secure_boot
        else:
            agent_pool_dict['security_profile'] = None

        if agent_pool.security_profile is not None:
            agent_pool_dict['security_profile']['enable_vtpm'] = agent_pool.security_profile.enable_vtpm
            agent_pool_dict['security_profile']['enable_secure_boot'] = agent_pool.security_profile.enable_secure_boot
        else:
            agent_pool_dict['security_profile'] = None

        if agent_pool.network_profile is not None:
            agent_pool_dict['network_profile'] = agent_pool.network_profile.as_dict()
        else:
            agent_pool_dict['network_profile'] = None

        if agent_pool.windows_profile is not None:
            agent_pool_dict['windows_profile']['disable_outbound_nat'] = agent_pool.windows_profile.disable_outbound_nat
        else:
            agent_pool_dict['windows_profile'] = None

        return agent_pool_dict


def main():
    AzureRMAksAgentPool()


if __name__ == '__main__':
    main()



# Copyright (c) 2024 Bill Peck  <bpeck@redhat.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_arcssh
version_added: "3.1.0"
short_description: Configure SSH proxy for ARC hosts
description:
    - Configure SSH proxy for ARC hosts. Use this in combination with the
      azure_rm inventory plugin with include_arc_resource_groups to dynamically
      generate the list of hosts.

options:
    resource_group:
        description:
            - The name of the Resource Group to which the server belongs.
        required: True
        type: str
    resource_type:
        description:
            - The name of the Resource Type to which the server belongs.
        required: True
        type: str
    inventory_hostname:
        description:
            - The inventory configured hostname.
        type: str
    ansible_host:
        description:
            - The discovered hostname that is generated by the inventory config.
            - Please see the examples on how this can be set based on the resource_type.
        type: str
    private_key_file:
        description:
            - The Private key that matches the public key that you have already added to your authorized_hosts
              of local_user.
        default: ~/.ssh/id_rsa
        type: str
    local_user:
        description:
            - The local user that you use to log into your server.
        type: str
    port:
        description:
            - The port that your ssh server is listening on.
        default: '22'
        type: str
    ssh_config_file:
        description:
            - Host specific file location for ssh config.  It must match the -F argument passed to
              ansible_ssh_common_args.  This will be overwritten and should be different for each host.
            - See the Examples for how to set this variable to match both the task and inventory plugin.
        required: True
        type: str
    ssh_relay_file:
        description:
            - Host specific file location for the relay file.
        required: True
        type: str
    ssh_proxy_folder:
        description:
            - The folder which will store the client ssh proxy.
        default: ~/.clientsshproxy
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
# Inventory plugin:
# ---
# plugin: azure.azcollection.azure_rm
# plain_host_names: yes
# include_arc_resource_groups: ['*']
#
#
# hostvar_expressions:
#   ansible_host: "resource_group + '-' + name if 'Microsoft.HybridCompute/machines' == resource_type else (public_dns_hostnames + public_ipv4_address) | first"
#   ansible_ssh_common_args: "'-F /tmp/' + resource_group + '-' + name + '/ssh_config' if 'Microsoft.HybridCompute/machines' == resource_type"
#
# keyed_groups:
# # places each host in a group named 'tag_(tag name)_(tag value)' for each tag on a VM.
#  - prefix: "type"
#    key: resource_type
#    trailing_separator: false

- name: Configure ARC SSH Proxy
  hosts: localhost
  connection: local
  tasks:
    - name: Setup Proxy
      azure.azcollection.azure_rm_arcssh:
        inventory_hostname: "{{ item }}"
        ansible_host: "{{ hostvars[item].ansible_host }}"
        local_user: admin
        resource_group: "{{ hostvars[item].resource_group }}"
        resource_type: "{{ hostvars[item].resource_type }}"
        private_key_file: "~/.ssh/id_rsa"
        ssh_config_file: "/tmp/{{ hostvars[item].resource_group }}-{{ item }}/ssh_config"
        ssh_relay_file: "/tmp/{{ hostvars[item].resource_group }}-{{ item }}/relay_info"
        ssh_proxy_folder: "~/.clientsshproxy"
      loop: "{{ groups['type_Microsoft_HybridCompute_machines'] }}"

- name: Ping ARC Hosts
  hosts: type_Microsoft_HybridCompute_machines
  tasks:
    - name: Ping
      ansible.builtin.ping:
'''



# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Nir Argaman, <nargaman@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class ModuleDocFragment(object):

    # Azure doc fragment
    DOCUMENTATION = r"""
options:
    identity:
        description:
            - Identity for this resource.
        type: dict
        suboptions:
            type:
                description:
                    - Type of the managed identity
                choices:
                    - SystemAssigned
                    - UserAssigned
                    - 'None'
                default: 'None'
                type: str
            user_assigned_identities:
                description:
                    - User Assigned Managed Identities and its options
                required: false
                type: dict
                default: {}
                suboptions:
                    id:
                        description:
                            - List of the user assigned IDs associated to this resource
                        required: false
                        type: list
                        elements: str
                        default: []
                    append:
                        description:
                            - If the list of identities has to be appended to current identities (true) or if it has to replace current identities (false)
                        required: false
                        type: bool
                        default: True
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage Check Point Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_threat_exception
short_description: Manages threat-exception objects on Check Point over Web Services API
description:
  - Manages threat-exception objects on Check Point devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R80 management version.
version_added: "1.0.0"
author: "Or Soffer (@chkp-orso)"
options:
  name:
    description:
      - The name of the exception.
    type: str
    required: True
  position:
    description:
      - Position in the rulebase. The use of values "top" and "bottom" may not be idempotent.
    type: str
  exception_group_uid:
    description:
      - The UID of the exception-group.
    type: str
  exception_group_name:
    description:
      - The name of the exception-group.
    type: str
  layer:
    description:
      - Layer that the rule belongs to identified by the name or UID.
    type: str
  rule_name:
    description:
      - The name of the parent rule.
    type: str
  action:
    description:
      - Action-the enforced profile.
    type: str
  destination:
    description:
      - Collection of Network objects identified by the name or UID.
    type: list
    elements: str
  destination_negate:
    description:
      - True if negate is set for destination.
    type: bool
  enabled:
    description:
      - Enable/Disable the rule.
    type: bool
  install_on:
    description:
      - Which Gateways identified by the name or UID to install the policy on.
    type: list
    elements: str
  protected_scope:
    description:
      - Collection of objects defining Protected Scope identified by the name or UID.
    type: list
    elements: str
  protected_scope_negate:
    description:
      - True if negate is set for Protected Scope.
    type: bool
  protection_or_site:
    description:
      - Name of the protection or site.
    type: list
    elements: str
  service:
    description:
      - Collection of Network objects identified by the name or UID.
    type: list
    elements: str
  service_negate:
    description:
      - True if negate is set for Service.
    type: bool
  source:
    description:
      - Collection of Network objects identified by the name or UID.
    type: list
    elements: str
  source_negate:
    description:
      - True if negate is set for source.
    type: bool
  track:
    description:
      - Packet tracking.
    type: str
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-threat-exception
  cp_mgmt_threat_exception:
    layer: New Layer 1
    name: Exception Rule
    position: 1
    protected_scope: All_Internet
    rule_name: Threat Rule 1
    state: present
    track: Log

- name: set-threat-exception
  cp_mgmt_threat_exception:
    layer: New Layer 1
    name: Exception Rule
    rule_name: Threat Rule 1
    state: present

- name: delete-threat-exception
  cp_mgmt_threat_exception:
    name: Exception Rule
    layer: New Layer 1
    rule_name: Threat Rule 1
    state: absent
"""

RETURN = """
cp_mgmt_threat_exception:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_objects,
    api_call,
    api_call_for_rule,
)


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        position=dict(type="str"),
        exception_group_uid=dict(type="str"),
        exception_group_name=dict(type="str"),
        layer=dict(type="str"),
        rule_name=dict(type="str"),
        action=dict(type="str"),
        destination=dict(type="list", elements="str"),
        destination_negate=dict(type="bool"),
        enabled=dict(type="bool"),
        install_on=dict(type="list", elements="str"),
        protected_scope=dict(type="list", elements="str"),
        protected_scope_negate=dict(type="bool"),
        protection_or_site=dict(type="list", elements="str"),
        service=dict(type="list", elements="str"),
        service_negate=dict(type="bool"),
        source=dict(type="list", elements="str"),
        source_negate=dict(type="bool"),
        track=dict(type="str"),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )
    api_call_object = "threat-exception"

    if module.params["position"] is None:
        result = api_call(module, api_call_object)
    else:
        result = api_call_for_rule(module, api_call_object)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2018, Mikhail Gordeev

# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: apt_repo
short_description: Manage APT repositories using C(apt-repo)
description:
  - Manages APT repositories using C(apt-repo) tool.
  - See U(https://www.altlinux.org/Apt-repo) for details about C(apt-repo).
notes:
  - This module works on ALT based distros.
  - Does NOT support checkmode, due to a limitation in C(apt-repo) tool.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  repo:
    description:
      - Name of the repository to add or remove.
    required: true
    type: str
  state:
    description:
      - Indicates the desired repository state.
    choices: [absent, present]
    default: present
    type: str
  remove_others:
    description:
      - Remove other then added repositories.
      - Used if O(state=present).
    type: bool
    default: false
  update:
    description:
      - Update the package database after changing repositories.
    type: bool
    default: false
author:
  - Mikhail Gordeev (@obirvalger)
"""

EXAMPLES = r"""
- name: Remove all repositories
  community.general.apt_repo:
    repo: all
    state: absent

- name: Add repository `Sisysphus` and remove other repositories
  community.general.apt_repo:
    repo: Sisysphus
    state: present
    remove_others: true

- name: Add local repository `/space/ALT/Sisyphus` and update package cache
  community.general.apt_repo:
    repo: copy:///space/ALT/Sisyphus
    state: present
    update: true
"""

RETURN = """ # """

import os

from ansible.module_utils.basic import AnsibleModule

APT_REPO_PATH = "/usr/bin/apt-repo"


def apt_repo(module, *args):
    """run apt-repo with args and return its output"""
    # make args list to use in concatenation
    args = list(args)
    rc, out, err = module.run_command([APT_REPO_PATH] + args)

    if rc != 0:
        module.fail_json(msg=f"'{' '.join(['apt-repo'] + args)}' failed: {err}")

    return out


def add_repo(module, repo):
    """add a repository"""
    apt_repo(module, "add", repo)


def rm_repo(module, repo):
    """remove a repository"""
    apt_repo(module, "rm", repo)


def set_repo(module, repo):
    """add a repository and remove other repositories"""
    # first add to validate repository
    apt_repo(module, "add", repo)
    apt_repo(module, "rm", "all")
    apt_repo(module, "add", repo)


def update(module):
    """update package cache"""
    apt_repo(module, "update")


def main():
    module = AnsibleModule(
        argument_spec=dict(
            repo=dict(type="str", required=True),
            state=dict(type="str", default="present", choices=["absent", "present"]),
            remove_others=dict(type="bool", default=False),
            update=dict(type="bool", default=False),
        ),
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    if not os.path.exists(APT_REPO_PATH):
        module.fail_json(msg="cannot find /usr/bin/apt-repo")

    params = module.params
    repo = params["repo"]
    state = params["state"]
    old_repositories = apt_repo(module)

    if state == "present":
        if params["remove_others"]:
            set_repo(module, repo)
        else:
            add_repo(module, repo)
    elif state == "absent":
        rm_repo(module, repo)

    if params["update"]:
        update(module)

    new_repositories = apt_repo(module)
    changed = old_repositories != new_repositories
    module.exit_json(changed=changed, repo=repo, state=state)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# cli_rm_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the module docstring and re-run
# cli_rm_builder.
#
#############################################

"""
The arg spec for the eos_logging_global module
"""


class Logging_globalArgs(object):  # pylint: disable=R0903
    """The arg spec for the eos_logging_global module"""

    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "config": {
            "type": "dict",
            "options": {
                "buffered": {
                    "type": "dict",
                    "options": {
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                            ],
                        },
                        "buffer_size": {"type": "int"},
                    },
                },
                "console": {
                    "type": "dict",
                    "options": {
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                            ],
                        },
                    },
                },
                "event": {
                    "type": "str",
                    "choices": [
                        "link-status",
                        "port-channel",
                        "spanning-tree",
                    ],
                },
                "facility": {
                    "type": "str",
                    "choices": [
                        "auth",
                        "cron",
                        "daemon",
                        "kern",
                        "local0",
                        "local1",
                        "local2",
                        "local3",
                        "local4",
                        "local5",
                        "local6",
                        "local7",
                        "lpr",
                        "mail",
                        "news",
                        "sys10",
                        "sys11",
                        "sys12",
                        "sys13",
                        "sys14",
                        "sys9",
                        "syslog",
                        "user",
                        "uucp",
                    ],
                },
                "format": {
                    "type": "dict",
                    "options": {
                        "hostname": {"type": "str"},
                        "timestamp": {
                            "type": "dict",
                            "options": {
                                "high_resolution": {"type": "bool"},
                                "traditional": {
                                    "type": "dict",
                                    "options": {
                                        "state": {
                                            "type": "str",
                                            "choices": ["enabled", "disabled"],
                                        },
                                        "timezone": {"type": "bool"},
                                        "year": {"type": "bool"},
                                    },
                                },
                            },
                        },
                        "sequence_numbers": {"type": "bool"},
                    },
                },
                "hosts": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "name": {"type": "str"},
                        "add": {"type": "bool"},
                        "remove": {"type": "bool"},
                        "protocol": {"type": "str", "choices": ["tcp", "udp"]},
                        "port": {"type": "int", "default": 514},
                    },
                },
                "level": {
                    "type": "dict",
                    "options": {
                        "facility": {"type": "str"},
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                            ],
                        },
                    },
                },
                "monitor": {"type": "str"},
                "turn_on": {"type": "bool"},
                "persistent": {
                    "type": "dict",
                    "options": {
                        "set": {"type": "bool"},
                        "size": {"type": "int"},
                    },
                },
                "policy": {
                    "type": "dict",
                    "options": {
                        "invert_result": {"type": "bool"},
                        "match_list": {"type": "str"},
                    },
                },
                "qos": {"type": "int"},
                "relogging_interval": {"type": "int"},
                "repeat_messages": {"type": "bool"},
                "source_interface": {"type": "str"},
                "synchronous": {
                    "type": "dict",
                    "options": {
                        "set": {"type": "bool"},
                        "level": {"type": "str"},
                    },
                },
                "trap": {
                    "type": "dict",
                    "options": {
                        "set": {"type": "bool"},
                        "severity": {
                            "type": "str",
                            "choices": [
                                "alerts",
                                "critical",
                                "debugging",
                                "emergencies",
                                "errors",
                                "informational",
                                "notifications",
                                "warnings",
                            ],
                        },
                    },
                },
                "vrfs": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "name": {"type": "str"},
                        "hosts": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "name": {"type": "str"},
                                "add": {"type": "bool"},
                                "remove": {"type": "bool"},
                                "protocol": {
                                    "type": "str",
                                    "choices": ["tcp", "udp"],
                                },
                                "port": {"type": "int", "default": 514},
                            },
                        },
                        "source_interface": {"type": "str"},
                    },
                },
            },
        },
        "running_config": {"type": "str"},
        "state": {
            "type": "str",
            "choices": [
                "deleted",
                "merged",
                "overridden",
                "replaced",
                "gathered",
                "rendered",
                "parsed",
            ],
            "default": "merged",
        },
    }  # pylint: disable=C0301



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_switch_routing_interfaces_info
short_description: Information module for devices _switch _routing _interfaces
description:
  - Get all devices _switch _routing _interfaces.
  - Get devices _switch _routing _interfaces by id.
  - List layer 3 interfaces for a switch. Those for a stack may be found under switch
    stack routing.
  - Return a layer 3 interface for a switch.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  serial:
    description:
      - Serial path parameter.
    type: str
  mode:
    description:
      - Mode query parameter. Optional parameter to filter L3 interfaces by mode.
    type: str
  protocol:
    description:
      - Protocol query parameter. Optional parameter to filter L3 interfaces by protocol.
    type: str
  interfaceId:
    description:
      - InterfaceId path parameter. Interface ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch getDeviceSwitchRoutingInterface
    description: Complete reference of the getDeviceSwitchRoutingInterface API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-device-switch-routing-interface
  - name: Cisco Meraki documentation for switch getDeviceSwitchRoutingInterfaces
    description: Complete reference of the getDeviceSwitchRoutingInterfaces API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-device-switch-routing-interfaces
notes:
  - SDK Method used are
    switch.Switch.get_device_switch_routing_interface,
    switch.Switch.get_device_switch_routing_interfaces,
  - Paths used are
    get /devices/{serial}/switch/routing/interfaces,
    get /devices/{serial}/switch/routing/interfaces/{interfaceId},
"""

EXAMPLES = r"""
- name: Get all devices _switch _routing _interfaces
  cisco.meraki.devices_switch_routing_interfaces_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    mode: string
    protocol: string
    serial: string
  register: result
- name: Get devices _switch _routing _interfaces by id
  cisco.meraki.devices_switch_routing_interfaces_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    serial: string
    interfaceId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "interfaceId": "string",
      "name": "string",
      "mode": "string",
      "subnet": "string",
      "interfaceIp": "string",
      "serial": "string",
      "switchPortId": "string",
      "multicastRouting": "string",
      "vlanId": 0,
      "uplinkV4": true,
      "uplinkV6": true,
      "ospfSettings": {
        "area": "string",
        "cost": 0,
        "isPassiveEnabled": true,
        "networkType": "string"
      },
      "ospfV3": {
        "area": "string",
        "cost": 0,
        "isPassiveEnabled": true,
        "networkType": "string"
      },
      "ipv6": {
        "assignmentMode": "string",
        "address": "string",
        "prefix": "string",
        "gateway": "string"
      },
      "vrf": {
        "name": "string"
      },
      "loopback": {},
      "defaultGateway": "string"
    }
"""



# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Akini Ross (@akinross) <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data, delete_none_values


class L3OutNode:
    def __init__(self, details, l3out_mso_template, l3out_object, pod_id, node_id):
        self.pod_id = pod_id
        self.node_id = node_id
        self.node_router_id = details.get("node_router_id")
        self.node_group_policy = details.get("node_group_policy")
        self.use_router_id_as_loopback = details.get("use_router_id_as_loopback")
        self.node_loopback_ip = details.get("node_loopback_ip")
        self.node = l3out_mso_template.get_l3out_node(l3out_object.details, self.pod_id, self.node_id)
        self.path = "/l3outTemplate/l3outs/{0}/nodes/{1}".format(l3out_object.index, self.node.index if self.node else "-")

    def construct_node_payload(self):
        return delete_none_values(
            {
                "group": self.node_group_policy,
                "podID": self.pod_id,
                "nodeID": self.node_id,
                "routerID": self.node_router_id,
                "useRouteIDAsLoopback": self.use_router_id_as_loopback,
                "loopbackIPs": [self.node_loopback_ip] if self.node_loopback_ip else None,
            }
        )

    def update_ops(self, ops):
        if self.node:
            self.set_node_replace_ops(ops)
        else:
            self.set_node_add_op(ops)

    def set_node_replace_ops(self, ops):
        remove_data = []
        node_payload = self.construct_node_payload()
        if node_payload.get("useRouteIDAsLoopback") is True or node_payload.get("loopbackIPs") == [""]:
            remove_data.append("loopbackIPs")
        append_update_ops_data(ops, self.node.details, self.path, node_payload, remove_data)

    def set_node_add_op(self, ops):
        ops.append(self.get_node_add_op())

    def get_node_add_op(self):
        return {"op": "add", "path": self.path, "value": self.construct_node_payload()}

    def get_node_remove_op(self):
        return {"op": "remove", "path": self.path}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_placement_group_info
version_added: 1.0.0
version_added_collection: community.aws
short_description: List EC2 Placement Group(s) details
description:
  - List details of EC2 Placement Group(s).
author:
  - "Brad Macpherson (@iiibrad)"
options:
  names:
    description:
      - A list of names to filter on. If a listed group does not exist, there
        will be no corresponding entry in the result; no error will be raised.
    type: list
    elements: str
    required: false
    default: []
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details or the AWS region,
# see the AWS Guide for details.

- name: List all placement groups.
  amazon.aws.ec2_placement_group_info:
  register: all_ec2_placement_groups

- name: List two placement groups.
  amazon.aws.ec2_placement_group_info:
    names:
      - my-cluster
      - my-other-cluster
  register: specific_ec2_placement_groups

- ansible.builtin.debug:
    msg: >
      {{ specific_ec2_placement_groups | json_query("[?name=='my-cluster']") }}
"""


RETURN = r"""
placement_groups:
  description: Placement group attributes.
  returned: always
  type: complex
  contains:
    name:
      description: PG name.
      type: str
      sample: "my-cluster"
    state:
      description: PG state.
      type: str
      sample: "available"
    strategy:
      description: PG strategy.
      type: str
      sample: "cluster"
    tags:
      description: Tags associated with the placement group.
      type: dict
      version_added: 8.1.0
      sample:
        tags:
          some: value1
          other: value2
"""

from typing import Any
from typing import Dict
from typing import List

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_ec2_placement_groups
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict


def get_placement_groups_details(connection, names: List) -> Dict[str, Any]:
    params = {}
    if len(names) > 0:
        params["Filters"] = [
            {
                "Name": "group-name",
                "Values": names,
            }
        ]
    response = describe_ec2_placement_groups(connection, **params)

    results = []
    for placement_group in response:
        results.append(
            {
                "name": placement_group["GroupName"],
                "state": placement_group["State"],
                "strategy": placement_group["Strategy"],
                "tags": boto3_tag_list_to_ansible_dict(placement_group.get("Tags")),
            }
        )
    return results


def main():
    argument_spec = dict(
        names=dict(type="list", default=[], elements="str"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    connection = module.client("ec2")
    names = module.params.get("names")

    placement_groups = get_placement_groups_details(connection, names)
    module.exit_json(changed=False, placement_groups=placement_groups)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Sabari Jaganathan (@sajagana) <sajagana@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_route_map_policy_route_control_context
version_added: "2.12.0"
short_description: Manage Route Map Policy for Route Control Context on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Route Map Policy for Route Control Context on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v4.1 and later.
author:
- Sabari Jaganathan (@sajagana)
options:
  template:
    description:
    - The name of the template.
    - The template must be a tenant template.
    - This parameter or O(template_id) is required.
    type: str
  template_id:
    description:
    - The ID of the tenant template.
    - This parameter or O(template) is required.
    type: str
  route_map_policy:
    description:
    - The name of the Route Map Policy for Route Control.
    - This parameter or O(route_map_policy_uuid) is required.
    type: str
  route_map_policy_uuid:
    description:
    - The UUID of the Route Map Policy for Route Control.
    - This parameter or O(route_map_policy) is required.
    type: str
  name:
    description:
    - The name of the Route Control context.
    type: str
  description:
    description:
    - The description of the Route Control context.
    - Providing an empty string O(description="") will remove description from the Route Control context.
    type: str
  order:
    description:
    - The order of the Route Control context.
    - The value must be a number between 0 and 9.
    - Defaults to C(0) when unset during creation.
    type: int
  action:
    description:
    - The action of the Route Control context.
    - Defaults to C(permit) when unset during creation.
    type: str
    choices: [ permit, deny ]
  set_rule:
    description:
    - The Set Rule Policy reference details for the Route Control context.
    - Providing an empty dictionary O(set_rule={}) will remove Set Rule Policy from the Route Control context.
    type: dict
    suboptions:
      uuid:
        description:
        - The UUID of the Set Rule Policy.
        - This parameter can be used instead of O(set_rule.reference).
        type: str
      reference:
        description:
        - The reference details of the Set Rule Policy.
        - This parameter can be used instead of O(set_rule.uuid).
        type: dict
        aliases: [ ref ]
        suboptions:
          name:
            description:
            - The name of the Set Rule Policy.
            type: str
          template:
            description:
            - The template associated with the Set Rule Policy.
            - This parameter or O(set_rule.reference.template_id) is required.
            type: str
          template_id:
            description:
            - The template ID associated with the Set Rule Policy.
            - This parameter or O(set_rule.reference.template) is required.
            type: str
  match_rules:
    description:
    - The list of Match Rule Policy references.
    - Providing an empty list O(match_rules=[]) will remove Match Rule Policies from the Route Control context.
    - When the O(match_rules) is null the update will not change existing configuration.
    - The old O(match_rules) list will be replaced by the new list during an update.
    type: list
    elements: dict
    suboptions:
      uuid:
        description:
        - The UUID of the Match Rule Policy.
        - This parameter can be used instead of O(match_rules.reference).
        type: str
      reference:
        description:
        - The reference of the Match Rule Policy.
        - This parameter can be used instead of O(match_rules.uuid).
        aliases: [ ref ]
        type: dict
        suboptions:
          name:
            description:
            - The name of the Match Rule Policy.
            type: str
          template:
            description:
            - The template associated with the Match Rule Policy.
            - This parameter or O(match_rules.reference.template_id) is required.
            type: str
          template_id:
            description:
            - The template ID associated with the Match Rule Policy.
            - This parameter or O(match_rules.reference.template) is required.
            type: str
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Tenant template.
- The O(route_map_policy) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_route_map_policy_route_control) to create the Route Map Policy for Route Control.
- The O(set_rule) must exist before using it with this module in your playbook.
  Use M(cisco.mso.ndo_set_rule_policy) to create the Set Rule Policy.
- The O(match_rules) must exist before using it with this module in your playbook.
  Use M(cisco.mso.ndo_match_rule_policy) to create the Match Rule Policy.
seealso:
- module: cisco.mso.ndo_template
- module: cisco.mso.ndo_route_map_policy_route_control
- module: cisco.mso.ndo_set_rule_policy
- module: cisco.mso.ndo_match_rule_policy
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add Route Map Policy for Route Control Context
  cisco.mso.ndo_route_map_policy_route_control_context:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    route_map_policy: ansible_test_route_map_policy
    name: route_map_policy_context_1
    order: 1
    set_rule:
      uuid: "{{ add_set_rule_policy_1.current.uuid }}"
    match_rules:
      - uuid: "{{ add_match_rule_policy_1.current.uuid }}"
      - reference:
          template_id: "{{ add_ansible_tenant_template.current.templateId }}"
          name: ansible_match_rule_policy_2
    state: present

- name: Update Route Map Policy for Route Control Context
  cisco.mso.ndo_route_map_policy_route_control_context:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    route_map_policy: ansible_test_route_map_policy
    name: route_map_policy_context_1
    order: 2
    set_rule:
      reference:
        template_id: "{{ add_ansible_tenant_template.current.templateId }}"
        name: ansible_set_rule_policy_2
    match_rules:
      - reference:
          template_id: "{{ add_ansible_tenant_template.current.templateId }}"
          name: ansible_match_rule_policy_2
    state: present

- name: Update and remove Set Rule and Match Rules from Route Map Policy for Route Control Context
  cisco.mso.ndo_route_map_policy_route_control_context:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    route_map_policy: ansible_test_route_map_policy
    name: route_map_policy_context_1
    order: 2
    set_rule: {}
    match_rules: []
    state: present

- name: Query Route Map Policy for Route Control Context using name
  cisco.mso.ndo_route_map_policy_route_control_context:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    route_map_policy: ansible_test_route_map_policy
    name: route_map_policy_context_1
    state: query
  register: query_one

- name: Query all Route Map Policy for Route Control Context
  cisco.mso.ndo_route_map_policy_route_control_context:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    route_map_policy: ansible_test_route_map_policy
    state: query
  register: query_one

- name: Delete Route Map Policy for Route Control Context
  cisco.mso.ndo_route_map_policy_route_control_context:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    route_map_policy: ansible_test_route_map_policy
    name: route_map_policy_context_1
    state: absent
"""

RETURN = r"""
"""

import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import KVPair
from ansible_collections.cisco.mso.plugins.module_utils.templates import MSOTemplates
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data, check_if_all_elements_are_none


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str"),
        template_id=dict(type="str"),
        route_map_policy=dict(type="str"),
        route_map_policy_uuid=dict(type="str"),
        name=dict(type="str"),
        description=dict(type="str"),
        order=dict(type="int"),
        action=dict(type="str", choices=["permit", "deny"]),
        set_rule=dict(
            type="dict",
            options=dict(
                uuid=dict(type="str"),
                reference=dict(
                    type="dict",
                    options=dict(
                        name=dict(type="str"),
                        template=dict(type="str"),
                        template_id=dict(type="str"),
                    ),
                    required_by={
                        "template": "name",
                        "template_id": "name",
                    },
                    aliases=["ref"],
                ),
            ),
        ),
        match_rules=dict(
            type="list",
            elements="dict",
            options=dict(
                uuid=dict(type="str"),
                reference=dict(
                    type="dict",
                    options=dict(
                        name=dict(type="str"),
                        template=dict(type="str"),
                        template_id=dict(type="str"),
                    ),
                    required_by={
                        "template": "name",
                        "template_id": "name",
                    },
                    aliases=["ref"],
                ),
            ),
        ),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["name"]],
            ["state", "absent", ["name"]],
        ],
        required_one_of=[
            ["template", "template_id"],
            ["route_map_policy", "route_map_policy_uuid"],
        ],
    )

    mso = MSOModule(module)
    mso_templates = MSOTemplates(mso)
    ops = []
    match = None
    path = None

    template_name = module.params.get("template")
    template_id = module.params.get("template_id")
    route_map_policy = module.params.get("route_map_policy")
    route_map_policy_uuid = module.params.get("route_map_policy_uuid")
    name = module.params.get("name")
    description = module.params.get("description")
    order = module.params.get("order")
    action = module.params.get("action")
    set_rule = module.params.get("set_rule")
    match_rules = module.params.get("match_rules")
    state = module.params.get("state")

    mso_template = mso_templates.get_template("tenant", template_name, template_id)
    mso_template.validate_template("tenantPolicy")
    route_map_policy_object = mso_template.get_route_map_policy(route_map_policy_uuid, route_map_policy, template_object=None, fail_module=True)
    reference_details = {
        "matchRule": {
            "name": "matchRuleName",
            "reference": "matchRuleRef",
            "type": "matchRule",
            "template": "matchRuleTemplateName",
            "templateId": "matchRuleTemplateId",
        },
        "setRule": {
            "name": "setRuleName",
            "reference": "setRuleRef",
            "type": "setRule",
            "template": "setRuleTemplateName",
            "templateId": "setRuleTemplateId",
        },
    }

    if name:  # Query a specific object
        match = mso_template.get_object_by_key_value_pairs(
            "Route Map Policy Route Control Context", route_map_policy_object.details.get("contexts", []), [KVPair("name", name)]
        )
        if match:
            if match.details.get("matchRules"):
                match.details["matchRules"] = match_rules_list_to_dict(match.details.get("matchRules"))
            mso.previous = mso.existing = mso_template.update_config_with_template_and_references(copy.deepcopy(match.details), reference_details, False)
    elif route_map_policy_object.details.get("contexts", []):  # Query all objects
        for obj in route_map_policy_object.details.get("contexts", []):
            if obj.get("matchRules"):
                obj["matchRules"] = match_rules_list_to_dict(obj.get("matchRules"))
            mso_template.update_config_with_template_and_references(obj, reference_details, False)
        mso.existing = route_map_policy_object.details.get("contexts", [])  # Query all

    if state != "query":
        path = "/tenantPolicyTemplate/template/routeMapPolicies/{0}/contexts/{1}".format(route_map_policy_object.index, match.index if match else "-")

    if state == "present":
        set_rule_uuid = None
        if set_rule:
            set_rule_uuid = set_rule.get("uuid")
            set_rule_reference = set_rule.get("reference")
            if not set_rule_uuid and set_rule_reference and not check_if_all_elements_are_none(set_rule_reference.values()):
                set_rule_template = mso_templates.get_template("tenant", set_rule_reference.get("template"), set_rule_reference.get("template_id"))
                set_rule_policy_match = set_rule_template.get_set_rule_policy_object(
                    uuid=None, name=set_rule_reference.get("name"), search_object=None, fail_module=True
                )
                set_rule_uuid = set_rule_policy_match.details.get("uuid")

        match_rule_uuids = []
        if match_rules:
            for match_rule in match_rules:
                if match_rule and match_rule.get("uuid"):
                    match_rule_uuids.append(match_rule.get("uuid"))
                elif match_rule and not check_if_all_elements_are_none(match_rule.get("reference", {}).values()):
                    ref = match_rule.get("reference")
                    match_rule_template = mso_templates.get_template("tenant", ref.get("template"), ref.get("template_id"))
                    match_rule_policy_match = match_rule_template.get_match_rule_policy_object(
                        uuid=None, name=ref.get("name"), search_object=None, fail_module=True
                    )
                    match_rule_uuids.append(match_rule_policy_match.details.get("uuid"))

        mso_values = {
            "name": name,
            "action": action,
            "description": description,
            "matchRules": match_rule_uuids,
            "setRuleRef": set_rule_uuid if set_rule_uuid else "",
        }

        if order is not None:
            mso_values["order"] = order

        if match:
            append_update_ops_data(ops, match.details, path, mso_values)
            mso.sanitize(mso_values, collate=True)
        else:
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=path, value=mso.sent))

    elif state == "absent" and match:
        ops.append(dict(op="remove", path=path))

    if mso.proposed:
        mso.proposed = copy.deepcopy(mso.proposed)
        mso.proposed["matchRules"] = match_rules_list_to_dict(mso.proposed.get("matchRules", []))
        proposed_reference_details = copy.deepcopy(reference_details)
        if mso.proposed.get("setRuleRef") == "":
            proposed_reference_details.pop("setRule", None)
        mso_template.update_config_with_template_and_references(mso.proposed, proposed_reference_details, False)

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        route_map_policy_object = mso_template.get_route_map_policy(
            route_map_policy_uuid, route_map_policy, template_object=response, fail_module=True
        ).details
        match = mso_template.get_object_by_key_value_pairs(
            "Route Map Policy Route Control Context", route_map_policy_object.get("contexts", []), [KVPair("name", name)]
        )
        if match:
            if match.details.get("matchRules"):
                match.details["matchRules"] = match_rules_list_to_dict(match.details.get("matchRules"))
            mso.existing = mso_template.update_config_with_template_and_references(match.details, reference_details, False)  # When the state is present
        else:
            mso.existing = {}  # When the state is absent

    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


def match_rules_list_to_dict(match_rules):
    return [{"matchRuleRef": mr} for mr in match_rules]


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Christian Kolrep <christian.kolrep@dataport.de>
# Copyright: (c) 2024, Akini Ross <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_epg
short_description: Manage End Point Groups (EPG) objects (fv:AEPg)
description:
- Manage End Point Groups (EPG) on Cisco ACI fabrics.
options:
  tenant:
    description:
    - Name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  ap:
    description:
    - Name of an existing application network profile, that will contain the EPGs.
    type: str
    aliases: [ app_profile, app_profile_name ]
  epg:
    description:
    - Name of the end point group.
    type: str
    aliases: [ epg_name, name ]
  bd:
    description:
    - Name of the bridge domain being associated with the EPG.
    type: str
    aliases: [ bd_name, bridge_domain ]
  priority:
    description:
    - The QoS class.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [ level1, level2, level3, unspecified ]
  intra_epg_isolation:
    description:
    - The Intra EPG Isolation.
    - The APIC defaults to C(unenforced) when unset during creation.
    type: str
    choices: [ enforced, unenforced ]
  description:
    description:
    - Description for the EPG.
    type: str
    aliases: [ descr ]
  fwd_control:
    description:
    - The forwarding control used by the EPG.
    - The APIC defaults to C(none) when unset during creation.
    type: str
    choices: [ none, proxy-arp ]
  preferred_group:
    description:
    - Whether or not the EPG is part of the Preferred Group and can communicate without contracts.
    - This is very convenient for migration scenarios, or when ACI is used for network automation but not for policy.
    - The APIC defaults to C(false) when unset during creation.
    type: bool
  monitoring_policy:
    description:
    - The name of the monitoring policy.
    type: str
  custom_qos_policy:
    description:
    - The name of the custom Quality of Service policy.
    type: str
  useg:
    description:
    - Use C(yes) to create uSeg EPG and C(no) is used to create Application EPG.
    type: str
    choices: [ 'yes', 'no' ]
  match:
    description:
    - The match type of the default Block Statement (fv:Crtrn).
    - The APIC defaults to C(any) when unset during creation.
    type: str
    choices: [ any, all ]
  precedence:
    description:
    - The Block Statement(fv:Crtrn) Precedence to resolve equal matches between micro segmented EPGs.
    - The APIC defaults to C(0) when unset during creation.
    type: int
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The C(tenant) and C(app_profile) used must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) and M(cisco.aci.aci_ap) modules can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_ap
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(fv:AEPg).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Swetha Chunduri (@schunduri)
- Shreyas Srish (@shrsr)
- Christian Kolrep (@Christian-Kolrep)
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Add a new EPG
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    ap: intranet
    epg: web_epg
    description: Web Intranet EPG
    bd: prod_bd
    monitoring_policy: default
    preferred_group: true
    state: present
  delegate_to: localhost

- name: Add a new uSeg EPG
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    ap: intranet
    epg: web_epg
    description: Web Intranet EPG
    bd: prod_bd
    monitoring_policy: default
    preferred_group: true
    useg: 'yes'
    state: present
  delegate_to: localhost

- name: Add a uSeg EPG with block statement match and precedence
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    ap: intranet
    epg: web_epg
    description: Web Intranet EPG
    bd: prod_bd
    monitoring_policy: default
    preferred_group: true
    useg: 'yes'
    match: all
    precedence: 1
    state: present
  delegate_to: localhost

- name: Query an EPG
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    ap: ticketing
    epg: web_epg
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all EPGs
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all EPGs with a Specific Name
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    validate_certs: false
    epg: web_epg
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all EPGs of an App Profile
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    validate_certs: false
    ap: ticketing
    state: query
  delegate_to: localhost
  register: query_result

- name: Remove an EPG
  cisco.aci.aci_epg:
    host: apic
    username: admin
    password: SomeSecretPassword
    validate_certs: false
    tenant: production
    app_profile: intranet
    epg: web_epg
    monitoring_policy: default
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        epg=dict(type="str", aliases=["epg_name", "name"]),  # Not required for querying all objects
        bd=dict(type="str", aliases=["bd_name", "bridge_domain"]),
        ap=dict(type="str", aliases=["app_profile", "app_profile_name"]),  # Not required for querying all objects
        tenant=dict(type="str", aliases=["tenant_name"]),  # Not required for querying all objects
        description=dict(type="str", aliases=["descr"]),
        priority=dict(type="str", choices=["level1", "level2", "level3", "unspecified"]),
        intra_epg_isolation=dict(choices=["enforced", "unenforced"]),
        fwd_control=dict(type="str", choices=["none", "proxy-arp"]),
        preferred_group=dict(type="bool"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
        monitoring_policy=dict(type="str"),
        custom_qos_policy=dict(type="str"),
        useg=dict(type="str", choices=["yes", "no"]),
        match=dict(type="str", choices=["all", "any"]),
        precedence=dict(type="int"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["ap", "epg", "tenant"]],
            ["state", "present", ["ap", "epg", "tenant"]],
        ],
    )

    aci = ACIModule(module)

    epg = module.params.get("epg")
    bd = module.params.get("bd")
    description = module.params.get("description")
    priority = module.params.get("priority")
    intra_epg_isolation = module.params.get("intra_epg_isolation")
    fwd_control = module.params.get("fwd_control")
    preferred_group = aci.boolean(module.params.get("preferred_group"), "include", "exclude")
    state = module.params.get("state")
    tenant = module.params.get("tenant")
    ap = module.params.get("ap")
    name_alias = module.params.get("name_alias")
    monitoring_policy = module.params.get("monitoring_policy")
    custom_qos_policy = module.params.get("custom_qos_policy")
    useg = module.params.get("useg")
    match = module.params.get("match")
    precedence = module.params.get("precedence")

    child_configs = [dict(fvRsBd=dict(attributes=dict(tnFvBDName=bd))), dict(fvRsAEPgMonPol=dict(attributes=dict(tnMonEPGPolName=monitoring_policy)))]

    if custom_qos_policy is not None:
        child_configs.append(dict(fvRsCustQosPol=dict(attributes=dict(tnQosCustomPolName=custom_qos_policy))))

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="fvAp",
            aci_rn="ap-{0}".format(ap),
            module_object=ap,
            target_filter={"name": ap},
        ),
        subclass_2=dict(
            aci_class="fvAEPg",
            aci_rn="epg-{0}".format(epg),
            module_object=epg,
            target_filter={"name": epg},
        ),
        child_classes=["fvRsBd", "fvRsAEPgMonPol", "fvRsCustQosPol", "fvCrtrn"],
    )

    aci.get_existing()

    if state == "present":
        if useg is not None and aci.existing and aci.existing[0]["fvAEPg"]["attributes"]["isAttrBasedEPg"] != useg:
            module.fail_json(msg="Changing attribute useg on existing EPG is not supported.")
        if useg == "yes":
            child_configs.append(dict(fvCrtrn=dict(attributes=dict(name="default", match=match, prec=precedence))))

        aci.payload(
            aci_class="fvAEPg",
            class_config=dict(
                name=epg,
                descr=description,
                prio=priority,
                pcEnfPref=intra_epg_isolation,
                fwdCtrl=fwd_control,
                prefGrMemb=preferred_group,
                nameAlias=name_alias,
                isAttrBasedEPg=useg,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="fvAEPg")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()
